# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
# 
#           SPVirtkeyboard.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
import logging
import random
import pygame
from pygame.constants import *

import SPWidgets
from SPConstants import *
import utils
import SPVirtkeyboardMap as VKM

KEYCOL = 45, 83, 152 

class SuperKey(SPWidgets.ImgButton):
    def __init__(self, s, pos, pad, value):
        SPWidgets.ImgButton.__init__(self, s, pos, pad, value)
    
class Key(SuperKey):
    def __init__(self, value, pos, obs, fsize=28, butsize=(48, 48)):
        ci = utils.char2surf(value,fsize, WHITE, ttf=TTFBOLD)
        s = pygame.Surface(butsize)
        s.fill(KEYCOL)
        pygame.draw.rect(s, WHITE, s.get_rect(), 1)
        cir = ci.get_rect()
        cir.center = s.get_rect().center
        s.blit(ci, cir)
        
        SuperKey.__init__(self, s, pos, 4, value)
        self.connect_callback(obs, MOUSEBUTTONUP, value)

class SpaceKey(SuperKey):
    def __init__(self, pos, obs, butsize=(100, 48)):
        s = pygame.Surface(butsize)
        s.fill(KEYCOL)
        pygame.draw.rect(s, WHITE, s.get_rect(), 1)
        
        SuperKey.__init__(self, s, pos, 4, 'space')
        self.connect_callback(obs, MOUSEBUTTONUP, 'space')

class BackspaceKey(SuperKey):
    def __init__(self, pos, obs, butsize=(100, 50)):
        s = pygame.Surface(butsize)
        s.fill(KEYCOL)
        
        pygame.draw.rect(s, WHITE, s.get_rect(), 1)
        pygame.draw.line(s, WHITE, (15,31), (92,31),2)
        pygame.draw.line(s, WHITE, (15,31), (20,26),2)
        pygame.draw.line(s, WHITE, (15,32), (20,37),2)
        
        SuperKey.__init__(self, s, pos, 4, 'backspace')
        self.connect_callback(obs, MOUSEBUTTONUP, 'backspace')

class EnterKey(SuperKey):
    def __init__(self, pos, obs, fsize=28, butsize=(100, 104)):
        text = utils.char2surf("Enter", fsize, WHITE, ttf=TTFBOLD)
        textpos = text.get_rect()
        s = pygame.Surface(butsize)
        s.fill(KEYCOL)
        sr = s.get_rect()
        blockoffx = (sr.width / 2)
        blockoffy = (sr.height / 2)
        offsetx = blockoffx - (textpos.width / 2)
        offsety = blockoffy - (textpos.height / 2) + 8 
        s.blit(text,(offsetx, (offsety-30)))
        
        pygame.draw.rect(s, WHITE, s.get_rect(), 1)
        pygame.draw.line(s, WHITE, (80,71), (80,81),2)
        pygame.draw.line(s, WHITE, (80,81), (25,81),2)
        pygame.draw.line(s, WHITE, (25,81), (30,76),2)
        pygame.draw.line(s, WHITE, (25,82), (30,87),2)
        
        SuperKey.__init__(self, s, pos, 4, 'enter')
        self.connect_callback(obs, MOUSEBUTTONUP, 'enter')


class ShiftKey(SuperKey):
    def __init__(self, pos, obs, fsize=28, butsize=(48, 48)):
        s = pygame.Surface(butsize)
        s.fill(KEYCOL)
        sr = s.get_rect()
        
        pygame.draw.rect(s, WHITE, s.get_rect(), 1)
        pygame.draw.line(s, WHITE, (22, 10), (22,36),2)
        pygame.draw.line(s, WHITE, (22, 10), (16,16),2)
        pygame.draw.line(s, WHITE, (22, 10), (28,16),2)
        
        SuperKey.__init__(self, s, pos, 4, 'shift')
        self.connect_callback(obs, MOUSEBUTTONUP, 'shift')

class KeyBoard(SPWidgets.Widget):
    """Class which provides a virtual on screen keyboard for use on touchscreens or
    in a kioskmode system.
    """
    def __init__(self, cbf, actives, keyboard_mode='qwerty', password_mode=False,\
                  wordcompletion=None, threshold=3, maxanswers=10, \
                  theme='default', position=(0, 100), \
                  display_position=(0, 0), maxlen=22, display_length=0, display=True):
        """This class provides a onscreen keyboard (670x225) with keys that are 
        placed in a SPGroup so that you can incorporate in your eventloop.
        The caller must call the update method on this group.
        The keyboard doesn't have an eventloop.
        The mandatory callback function must be able receive the following data from this object:
            self and a string with the text entered by the user when the user hits Enter.
            self and a list with strings from the wordcompleter if used.
        After Enter is clicked the widget is still fully functional.
        
        cbf - The callback function to call when the user hits enter.
        actives - A SPGroup instance.
        keyboard_mode - Define layout, possible values: 'qwerty', 'qwerty_square',
                    'numbers', 'abc','abc_square'
        password_mode - Boolean. Characters entered will be displayed as stars.
        wordcompletion - WordCompleter instance or None. (only used if threshold > 0)
        threshold - Threshold value for wordcompletion. (0 means no wordcompletion) 
        maxanswers - The maximum number of completion suggestions. (defaults to 10)
        theme - Theme used. Values taken from SPData/base/themes/core.rc
        display_position - position in x,y coords for the text display.
        display_length - length of the display in pixels. 0 means equal to the keyboard length.
        maxlen - number of characters to display.
        display - If normal display is to be used. When set to None the entry which has
         the input focus. You must make sure to set some entry widget to have the input focus
         otherwise nothing is shown.
        """
        SPWidgets.Widget.__init__(self)
        self.logger = logging.getLogger("childsplay.SPVirtkeyboard.Keyboard")
        self.cbf = cbf
        self.position = position
        self.password_mode = password_mode
        self.maxlen = maxlen
        self.display_length = display_length
        self.display_position = display_position
        self.actives = actives
        self.display = display
        self._ShiftActive = False
        self._KeepDisplay = False
        self.keyboard_mode = keyboard_mode
        self.set_keymap()
        # wordcompleter related stuff.
        if threshold == 0:
            self.WC = None
        else:
            self.WC = wordcompletion
        self.threshold = threshold
        self.text = ''# we also keep text in TI but local is faster (for wordcompletion)
        self.maxanswers = maxanswers
        self._IamShowed = False
                
    def set_keymap(self):
        kbmap = self.keyboard_mode
        if kbmap == 'qwerty':
            map = VKM.Qwerty()
        elif kbmap == 'abc':
            map = VKM.Abc()
        elif kbmap == 'numbers':
            map = VKM.Numbers()
        elif kbmap == 'qwerty_square':
            map = VKM.QwertySquare()
        elif kbmap == 'abc_square':
            map = VKM.AbcSquare()
        elif kbmap == 'abc_minus':
            map = VKM.AbcMinus()
        elif kbmap == 'qwerty_minus':
            map = VKM.QwertyMinus()
        but_x,  but_y = 56, 56
                # setup keys
        x = self.position[0]
        y = self.position[1]
        extrakeys_pos = []
        self.keys = []
        for line in map.getlines():
            for k in line:
                if self._ShiftActive:
                    k = k.upper()
                else:
                    k = k.lower()
                k = Key(k,(x, y), self.observer)
                self.keys.append(k)
                x += but_x
            extrakeys_pos.append((x, y))
            x = self.position[0]
            y += but_y
        # bacspace
        kb = BackspaceKey(extrakeys_pos[0], self.observer)
        self.keys.append(kb)
        
        # enter
        ke = EnterKey(extrakeys_pos[1], self.observer)
        self.keys.append(ke)
        
        # space
        if map != 'numbers':
            ks = SpaceKey(extrakeys_pos[3], self.observer)
            ksh = ShiftKey((extrakeys_pos[3][0] + ks.rect.w, extrakeys_pos[3][1]),\
                                            self.observer)
            self.keys += [ks, ksh]
            
        correction = 8
        if self.display_length == 0:
            self.display_length = extrakeys_pos[0][0] - x + kb.get_sprite_width() - correction    
        # setup the text display
        if self._KeepDisplay:
            return
        if self.display:
            self.TI = SPWidgets.TextEntry(pos=self.display_position, \
                            length=self.display_length, maxlen=self.maxlen, \
                            message='', fsize = 28, border=True, ttf=TTFBOLD, \
                            fgcol=BLACK, password_mode=self.password_mode)
            self.actives.set_input_focus(self.TI)
        else:
            self.TI = self.actives.who_has_input_focus()
            self._KeepDisplay = True

    def _set_firstletter(self, c, snd):
        # special attributes set in TextEntry, only used by various Argos acts.
        self.TI._fl = c.upper()
        self.TI._snd = snd
        
    def show(self):
        for b in self.keys:
            b.display_sprite()
        self.TI.display_sprite()
        if self._IamShowed:
            return
        else:
            self.actives.add(self.keys)
            if not self._KeepDisplay:
                self.actives.add(self.TI)
            self._IamShowed = True
        
    def hide(self):
        for b in self.keys:
            b.erase_sprite()
        if  not self._KeepDisplay:
            self.TI.erase_sprite()
        if not self._IamShowed:
            return
        else:
            self.actives.remove(self.keys)
            if not self._KeepDisplay:
                self.actives.remove(self.TI)
            self._IamShowed = False
    
    def visible(self):
        return self._IamShowed
        
    def destroy(self):
        self.clear_display()
        self.hide()
    
    def clear_display(self):
        self.TI.clear()
    
    def observer(self, widget, event, data):
        """Notified by the key class"""
        if self.TI is not self.actives.who_has_input_focus():
            self.TI = self.actives.who_has_input_focus()
        c = data[0]
        if c == 'space':
            c = ' '
            self.TI.add(c)
        elif c == 'backspace':
            self.TI.backspace()
            text = self.TI.get_text().lower()
            if self.WC and len(text) >= self.threshold:
                ans = self.WC.get_answers(text, self.maxanswers)
                self.cbf(self, ans)
        elif c == 'enter':
            # notify our parent observer
            result = self.cbf(self, self.get_text())
            if result:
                self.hide()
        elif c == 'shift':
            self._KeepDisplay = True
            if self._ShiftActive:
                self._ShiftActive = False
                self.hide()
                self.set_keymap()
                self.show()
            else:
                self._ShiftActive = True
                self.hide()
                self.set_keymap()
                self.show()
        else:
            if self._ShiftActive:
                self._ShiftActive = False
                self.hide()
                self.set_keymap()
                self.show()
            self.TI.add(c)
            text = self.TI.get_text().lower()
            if self.WC and len(text) >= self.threshold:
                ans = self.WC.get_answers(text, self.maxanswers)
                self.cbf(self, ans)
                
    def get_text(self):
        return self.TI.get_text()
        
    def toggle_numbers(self, value):
        pass
        
        
class WordCompleter:
    """Class which provides word completion.
    """
    def __init__(self, wordlist, firstletter=''):
        """All the words from the list are converted to lowercase and doubles
        are removed.
        wordlist - list with strings
        firstletter - one character string indicating for which letter we look for suggestions."""
        if firstletter:
            self._firstletter = firstletter.lower()
        else:
            self._firstletter = None
        self.set_wordlist(wordlist)
    
    def set_wordlist(self, wordlist):
        self.words = {}
        self.wordlist = wordlist
        for w in wordlist:
            w = w.lower()
            if not self.words.has_key(w[0]):
                self.words[w[0]] = [w]
            else:
                self.words[w[0]].append(w)
    
    def set_firstletter(self, c):
        if not c:
            self._firstletter = None
        else:
            self._firstletter = c.lower()
    
    def get_answers(self, part, max=0):
        """Get suggestions from the wordlist based on the string @part.
        part is the partial string.
        max - the number of possible answers we should return.
        More answers means more lookup time.
        Most of the time 10 answers is more than enough.
        0 means all answers"""
        answers = []
        #print self._firstletter
        if not self._firstletter:
            self._firstletter = part[0].lower()
        if part[0].lower() != self._firstletter:
            return []
        try:
            for k in self.words[part[0].lower()]:
                if k.startswith(part):
                    if k[-1] == '\n':
                        k = k[:-1]
                    answers.append(k)
        except KeyError:
            answers = []
        else:
            if max:
                try:
                    foundanswers = random.sample(answers, max)
                except ValueError:
                    pass
                else:
                    return foundanswers
            return answers
        return []

    def lookup(self, word):
        """Check to see if @word is part of the wordlist.
        The words in the wordlist must end with a \n, not \r\n
        Returns True of False."""
        if not word or not self._firstletter:
            return None
        if 47 < ord(word[0]) < 58:
            # The first letter is a number 
            return True
        return word+'\n' in self.words[self._firstletter]


if __name__ == '__main__':
    
    import __builtin__
    __builtin__.__dict__['_'] = lambda x:x
    
    import SPLogging
    SPLogging.set_level('debug')
    SPLogging.start()
    import psyco
    psyco.full()
#    f = open('dictionary_nl.txt', 'r')
#    wordlist = f.readlines()
#    f.close()
    wordlist = []
    wc = WordCompleter(wordlist)
    print wc.get_answers('aa')
    
    import pygame
    from pygame.constants import *
    pygame.init()
    
    from SPSpriteUtils import SPInit
    
    scr = pygame.display.set_mode((800, 600))
    scr.fill((135,206,250))
    
    back = scr.convert()
    s = utils.char2surf("Hit ESC to cycle trough the layouts, F1 to quit", 24)
    scr.blit(s, (20, 10))
    
    actives = SPInit(scr, back)
    
    pygame.display.flip() 
    
    SPWidgets.Init('braintrainer')
    
    def cbf(parent, data):
        print "cbf called with: ", parent, data
    
    kb = None
    for layout in ('qwerty', 'qwerty_square','numbers', 'abc', 'abc_square'):
        if kb:
            kb.hide()
        actives.empty()
        kb = KeyBoard(cbf, actives, keyboard_mode=layout, position=(80, 210), \
                      wordcompletion=wc, threshold=2, \
                      display_position=(80, 160), display_length=0, \
                      password_mode=False)
        kb.show()
        runloop = 1
        while runloop:
            pygame.time.wait(100)
            pygame.event.pump()
            events = pygame.event.get()
            for event in events:
                if event.type is KEYDOWN:
                    if event.key == K_ESCAPE:
                        runloop = 0
                    elif event.key == K_F1:
                        sys.exit(0)
                else:
                    actives.update(event)
    




# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
# Parts are taken from the Braintrainer project http://www.braintrainerplus.com
#
#           simon_sp.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# TODO: this has to be rewritten some day to make it more SP like.
# Major headache is the stuff that goes on in the eventloop.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.simon_sp.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.simon_sp")

# standard modules you probably need
import os,sys,random

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils

CYELLOW = 1
CRED = 2
CBLUE = 3
CGREEN = 4

class CenterButton(SPSpriteUtils.SPSprite):
    def __init__(self, img, pos):
        SPSpriteUtils.SPSprite.__init__(self, img)
        self.rect = self.image.get_rect()
        self.moveto(pos)
        
class SndButton(SPSpriteUtils.SPSprite):
    def __init__(self, img_off, img_on, snd,pos, obs, col):
        SPSpriteUtils.SPSprite.__init__(self, img_off)
        self.rect = self.image.get_rect()
        self.obs = obs
        self.image_on = img_on
        self.image_off = img_off
        self.col = col
        self.snd = snd
        self.connect_callback(self.cbf, event_type=MOUSEBUTTONDOWN)
        self.moveto(pos)
    
    def activate(self):
        self.snd.play()
        self.erase_sprite()
        self.image = self.image_on
        self.display_sprite()
        pygame.time.delay(600)
        self.erase_sprite()
    
    def cbf(self, *args):
        # check for alpha pixel and if not display light image and call obs 
        mx, my = pygame.mouse.get_pos()
        pos = (mx - self.rect.left, my - self.rect.top)
        c = self.image.get_at(pos)[3]
        if c:
            self.activate()
            self.obs(self.col)
        return -1
            
        
class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.simon_sp.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.theme = self.SPG.get_theme()
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','Simon_spData')
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'simon_sp.rc'))
        self.rchash['theme'] = self.theme
        
        self.logger.debug("found rc: %s" % self.rchash)
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
        
        ### BT related stuff, was rc file dependant replaced by hardcoded values.
        if self.blit_pos[1] == 0:
            y =10
        else:
            y = 110
        self.yTop=y #int(self.config["yMargin"])
        self.xMargin=0  #int(self.config["xMargin"])
        ##################################
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.sequence = []
        self.all_choices=[1,2,3,4]
        
        
    def get_moviepath(self):
        movie = os.path.join(self.my_datadir,'help.avi')
        return movie
    
    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.redraw()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Simon_sp")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "simon_sp"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
        _("Try to repeat the sequence of sounds played."),
        " ", 
        _("On the screen are four blocks of different colours."), 
        _("Press the Start button with your finger."), 
        " ", 
        _("The  blocks will light up randomly along with sound clues."), 
        _("When the flashing stops you will be prompted to repeat the sequence by touching the coloured blocks."), 
        " ",
        ]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return _("Mind the sound sequence as well as the colors sequence.")
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Memory")
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6
    
    def stop(self):
        pygame.event.clear()
        
    def start(self):
        """Mandatory method."""
        self.snd_butts = []
        backdir = os.path.join(self.my_datadir,'backgrounds', self.lang)
        if not os.path.exists(backdir):
            backdir = os.path.join(self.my_datadir,'backgrounds', 'en')
        yellow_sound = utils.load_sound(os.path.join(self.my_datadir,'1.ogg'))
        red_sound = utils.load_sound(os.path.join(self.my_datadir,'2.ogg'))
        blue_sound = utils.load_sound(os.path.join(self.my_datadir,'3.ogg'))
        green_sound = utils.load_sound(os.path.join(self.my_datadir,'4.ogg'))
        self.good_sound=utils.load_sound(os.path.join(self.my_datadir,'good.ogg'))
        self.wrong_sound=utils.load_sound(os.path.join(self.my_datadir,'wrong.ogg'))
        self.backgr_normal = utils.load_image(os.path.join(backdir,'background.png'))
        self.backgr_start = utils.load_image(os.path.join(backdir,'background_start.png')) 
        self.backgr_tussen = utils.load_image(os.path.join(backdir,'background_repeat.png'))
        self.backgr_good = utils.load_image(os.path.join(backdir,'background_correct.png'))
        self.backgr_wrong = utils.load_image(os.path.join(backdir,'background_wrong.png'))
        
        yellow_on = utils.load_image(os.path.join(self.my_datadir,'yellow.png'))
        yellow_off = utils.load_image(os.path.join(self.my_datadir,'yellow_off.png')) 
        self.yellow_but = SndButton(yellow_off, yellow_on, yellow_sound,(14,245+self.yTop),\
                                     self.snd_but_observer, CYELLOW)   
        #self.yellow_but.set_use_current_background(True)
        self.snd_butts.append(self.yellow_but)    

        red_on = utils.load_image(os.path.join(self.my_datadir,'red.png'))
        red_off = utils.load_image(os.path.join(self.my_datadir,'red_off.png'))
        self.red_but = SndButton(red_off, red_on, red_sound,(400,16+self.yTop), \
                                 self.snd_but_observer, CRED)
        #self.red_but.set_use_current_background(True)
        self.snd_butts.append(self.red_but)
        
        blue_on = utils.load_image(os.path.join(self.my_datadir,'blue.png'))
        blue_off = utils.load_image(os.path.join(self.my_datadir,'blue_off.png'))
        self.blue_but = SndButton(blue_off, blue_on, blue_sound,(14,16+self.yTop),\
                                   self.snd_but_observer, CBLUE)
        #self.blue_but.set_use_current_background(True)
        self.snd_butts.append(self.blue_but)
        
        green_on = utils.load_image(os.path.join(self.my_datadir,'green.png'))
        green_off = utils.load_image(os.path.join(self.my_datadir,'green_off.png')) 
        self.green_but = SndButton(green_off, green_on, green_sound, (400,245+self.yTop),\
                                    self.snd_but_observer, CGREEN)
        #self.green_but.set_use_current_background(True)
        self.snd_butts.append(self.green_but)

        langdatadir = os.path.join(self.my_datadir,self.lang)
        if not os.path.exists(langdatadir):
            langdatadir = os.path.join(self.my_datadir, 'en')
        img = utils.load_image(os.path.join(langdatadir,'start.png'))
        self.start_but = CenterButton(img, (232, 148+self.yTop))
        self.start_but.connect_callback(self._cbf_start_button, MOUSEBUTTONDOWN)
        
        img = utils.load_image(os.path.join(langdatadir,'simon.png'))
        self.simon_but = CenterButton(img, (232, 148+self.yTop))
        img = utils.load_image(os.path.join(langdatadir,'good.png'))
        self.good_but = CenterButton(img, (232, 148+self.yTop))
        img = utils.load_image(os.path.join(langdatadir,'wrong.png'))
        self.wrong_but = CenterButton(img, (232, 148+self.yTop))      
        # lookup hash for the sounds dbase field, see generate_sequence for the
        # number of sounds in each level
        self.level_sounds_hash = {1:2, 2:3, 3:3, 4:4, 5:5, 6:6}
        self.scoredisplay.clear_score()
        self.AreWeDT = False
        self.score = 0
       
    def snd_but_observer(self,col):
        self.clicked_colors.append(col)
        self.check_sequence()
                
    def _cbf_start_button(self, widget,event,data):
        self.logger.debug("start clicked")
        mx, my = pygame.mouse.get_pos()
        pos = (mx - widget.rect.left, my - widget.rect.top)
        c = widget.image.get_at(pos)[3]
        if c:
            widget.erase_sprite()
            self.actives.remove(widget)
            self.start_sequence()
            self.actives.add(self.snd_butts)
            pygame.event.clear()
                
    def dailytraining_pre_level(self, level):
        """Mandatory method"""
        pass
        
    def dailytraining_next_level(self, level, dbmapper):
        """Mandatory method.
        This should handle the DT logic"""
        self.AreWeDT = True
        self.SPG.tellcore_disable_level_indicator()
        self.next_level(level, dbmapper)
        return True
    
    def pre_level(self, level):
        """Mandatory method"""
        pass

    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("next_level called with %s" % level)
        if level > 6: return False # We only have 6 levels
        self.dbscore = 0
        # db_mapper is a Python class object that provides easy access to the 
        # dbase.
        self.dbmapper = dbmapper
        # number of exercises in one level
        if self.AreWeDT:
            self.exercises = 1
        else:
            self.exercises = int(self.rchash[self.theme]['exercises'])
        self.level=level
        self.levelupcount = 1
        self.dbmapper.insert('sounds', self.level_sounds_hash[level])
        self.stop_level = False
        self.levelrestartcounter = 0
        self.actives.empty()
        self.start_exercise()
        return True
    
    def start_exercise(self):
        self.levelrestartcounter += 1
        self.clicked_colors=[]
        # reset the screen to clear any crap from former levels
        self.clear_screen()
        self.screen.blit(self.backgr_normal, (15,self.yTop+15))
        self.backgr.blit(self.backgr_normal, (15,self.yTop+15))
        pygame.display.update()
        pygame.time.wait(1000)
        self.start_but.display_sprite()
        self.actives.add(self.start_but)
    
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        pass 
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        m,s = timespend.split(':')
        seconds = int(m)*60 + int(s)
        # score should be dependent on how many correct sequence parts are entered.
        # N = Number sequence parts * number of exercises 
        # P = Users result in points
        # F = Fraction of total result related to points. (maximum result is 10)
        # F1 = Fraction of result related to time
        # C = multiplier
        # S = time spend in seconds
        N = float(self.level_sounds_hash[self.level] * self.exercises)
        P = float(self.dbscore)
        F = 8.0
        F1 = 2.0
        C = 0.4
        C1 = 1.3
        S = float(seconds)
        points = max((F/100) * ((P/(N/100)) * C1), 1.0)
        time = min(F1 / ((S/(N*2)) ** C), F1)
        self.logger.info("@scoredata@ %s level %s N %s P %s S %s points %s time %s" %\
                          (self.get_name(),self.level, N, P, S, points, time))
        result = points + time
        return result

    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass
    
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()
                                
    def check_sequence(self):
        l = len(self.clicked_colors)
        if self.sequence[:l] != self.clicked_colors[:l]:
            self.logger.debug("Bad sequence entered")
            self.actives.remove(self.snd_butts)
            self.score = max(0, (self.score - 50)) 
            if self.levelupcount > 0: 
                self.levelupcount -= 1
            self.clicked_colors=[]
            self.screen.blit(self.backgr_wrong,(0+15+self.xMargin,0+self.yTop+15))
            pygame.display.update()
            self.wrong_sound.play()
            pygame.time.delay(2000)
            pygame.event.clear()
            self.start_exercise()
        if len(self.clicked_colors) == len(self.sequence) and self.sequence == self.clicked_colors:
            self.logger.debug("sequence correct")
            self.actives.remove(self.snd_butts)
            self.score += 50
            self.levelupcount += 1
            self.scoredisplay.set_score(self.score*2)
            self.dbscore += len(self.sequence)
            self.screen.blit(self.backgr_good,(0+15+self.xMargin,0+self.yTop+15))
            pygame.display.update()
            self.good_sound.play()
            pygame.time.delay(2000)
            pygame.event.clear()
            self.start_exercise()
        

    def start_sequence(self):    
        self.wrong_sequence = False
        self.sequence=self.generate_sequence()
        self.logger.debug("generated sequence:%s" % self.sequence)
        self.play_sequence()
        
    def play_sequence(self):
        pygame.time.delay(1000)
        self.logger.debug("Playing sequence")
        for color in self.sequence:
            if color==CYELLOW: self.yellow_but.activate()
            if color==CRED: self.red_but.activate()
            if color==CBLUE: self.blue_but.activate()
            if color==CGREEN: self.green_but.activate()
            pygame.time.delay(500)
        self.sequence_index = 0
        pygame.event.clear()
        self.screen.blit(self.backgr_tussen, (15,self.yTop+15))
        self.backgr.blit(self.backgr_tussen, (15,self.yTop+15))
        pygame.display.update()
        
        
    def generate_sequence(self):
        unique_choices=[]
        if self.level==1:
            unique_choices = random.sample(self.all_choices,2)
            return unique_choices
        if self.level==2:
            #3 choices: 2 unique choices and 1 double
            unique_choices = random.sample(self.all_choices,2)
            append_nr=random.sample(unique_choices,1)
            unique_choices.append(append_nr[0])
            return unique_choices
        if self.level==3:
            #3 unique choices
            unique_choices = random.sample(self.all_choices,3)
            return unique_choices
        if self.level==4:
            #4choices: 3 unique choices and 1 double
            unique_choices = random.sample(self.all_choices,3)
            append_nr=random.sample(unique_choices,1)
            unique_choices.append(append_nr[0])
            return unique_choices
        else:
            # levelNr complete random choices 
            i=0
            while i < self.level:
                append_nr=random.sample(self.all_choices,1)
                unique_choices.append(append_nr[0])
                i+=1
            return unique_choices

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        for event in events:
            if event.type is MOUSEBUTTONDOWN:
                self.actives.update(event)
        if self.levelrestartcounter >= self.exercises:
            if self.levelrestartcounter >= int(self.rchash[self.theme]['autolevel_value']):
                levelup = 1
            else:
                levelup = 0
            if self.AreWeDT:
                self.SPG.tellcore_level_end(level=self.level)
            else:
                self.SPG.tellcore_level_end(store_db=True, \
                                    level=min(6, self.level), \
                                                levelup=levelup)



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPSpriteUtils.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""
SPSpriteUtils - Module which provides some high level extensions to the 
basic pygame.sprite classes.

class SPSprite - A class which extends the standard pygame Sprite class.

class SPGroup - A class which extends the standard pygame RenderUpdate group class.

You should let your classes inherit from these classes.
For a detailed explanation you should check the pygame docs as we just extends
that class.
Also look at the docstrings in these classes for more info.
"""

import math
import sys
import os
import logging
import pygame
import types
from pygame.constants import *
from types import StringType, ListType

from utils import MyError, char2surf, load_image
from SPConstants import ACTIVITYDATADIR, BUTTON_FEEDBACK_TIME
module_logger = logging.getLogger("childsplay.SPSpriteUtils_lgpl")

def SPInit(scr, back, cmd_options={}):
    """ Init(back,scr,group=pygame.sprite.Group()) -> SPGroup instance
        
         This MUST be called BEFORE the SpriteUtils classes are used, and is needed
         for some of the magic.
         It's very straight forward and can probably used for all plugins.
         It must be called with references to the background surface and display surface.
         back = background surface.
         scr = display surface.
         
         group = pygame.sprite group used as the group which holds all the sprites.
         You probably don't want to change this group.
         
         theme  = The them used. Optional only needed for the GUI stuff from SPSpriteGui.
         The group instance returnt is a extended pygame.sprite group, which you should
         use to put your sprites in and then call the refresh method to update all the
         sprites. Be sure to check the reference before using this high level classes.
         It also possible to call this function multiple times to get multiple classes, but be aware
         that they all share the same screen and background surfaces. 
         """
    
    logger = logging.getLogger("childsplay.SPSpriteUtils_lgpl.SPInint")
    
    SPSprite.group = pygame.sprite.Group()# base class to hold sprites
    # All the objects which derives from SPSprite belongs to this group.
    # store a reference, needed for removal and erasing of sprites
    SPSprite.backgr = back
    SPSprite.screen = scr
    for k,v in cmd_options.items():
        setattr(SPSprite, "_Init_cmd_opt_%s" % k, v)
    # SPGroup needs screen and backgr to erase and draw the sprites
    # belonging to this group
    g = SPGroup(scr, back)# special CP group
    return g

class SPSprite(pygame.sprite.Sprite):
    """ Class to inherited from by regular sprite classes.
     When you extend this class you must call the constructor *before* you alter
     the 'rect' member. 
     
     This class calls the pygame.sprite.Sprite constructor and adds itself
     to the RenderUpdates group.
     There's also the possibility to connect a callback method to a pygame event
     much like the GTK/libglade way of callbacks.
     
     Be aware that this class derives from the pygame Sprite class, so
     there are two mandatory attributes: 'image' and 'rect'.
     See the pygame.sprite.Sprite reference for more information.
     
     You MUST call the init function BEFORE you use anything in this module.
    """
    def __init__(self, image=None, name=None):
        """Turn an image into a sprite object.
        image - pygame.Surface. If set to None you must set the image and rect
                attributes before using it. Only set it to None if you know
                why you want it too be None.
        name - a string so that one can do: if sprite_obj == 'foobar':"""
        pygame.sprite.Sprite.__init__(self)# This must be set before using this class
        self._logger = logging.getLogger("childsplay.SPSpriteUtils_lgpl.SPSprite")
        self.DEBUG = 0
        
        # make sure we have a correct background
        #self.backgr = self.backgr.convert()
        #self.backgr.blit(self.screen, (0, 0))
        self.image = image
        if self.image:
            self.rect = image.get_rect()
        else:
            self.rect = pygame.Rect(0,0,0,0)
        self.hover_active = False
        self._args = [self] # default value to pass to update functions
        self._event_type = [] # default in case there's no callback while there's a mouseevent
        self._callback = None # same as with self.event_type
        # defaults for movement of the object
        self._start = (0, 0)
        self._end = (0, 0)
        self._vector = (0, 0)
        self.__name = name
        self.name = name
        self._moveit_iter = None
        
        # a group that owns this sprite, used to address common properties
        # of sprites within the group
        self.group_owner = None
        self.UseCurrentScreenAsBackground = False
        self.erasesurf = None
        # DnD
        self._org_pos = self.rect.topleft
            # needed for rotation
        self.__dir = 0
        self.__OrgImage = self.image #comes from the derived class
        
    def rotate_sprite(self, amount):
        """ Rotate the sprite by the amount given. The image should be loaded
        without setting aplha channel otherwise pygame will crash!!
        See utils.load_image for arguments to unset alphachannel.
        
        Positive degrees rotates counterclockwise.
        Negative degrees rotates clockwise"""
        oldCenter = self.rect.center
        self.__dir += amount
        if self.__dir >= 360:
            self.__dir = 0
            self.image = self.__OrgImage
        self.undraw_sprite()
        self.image = pygame.transform.rotate(self.__OrgImage, self.__dir)
        self.rect = self.image.get_rect()
        self.rect.center = oldCenter
        self.draw_sprite()
        
    def draw_sprite(self,pos=None): 
        """Draw the sprite on the screen surface
        """
        if self.UseCurrentScreenAsBackground:
            #self.backgr = pygame.display.get_surface()
            self.erasesurf = pygame.Surface(self.image.get_size())
            self.erasesurf.blit(pygame.display.get_surface(), (0, 0), self.rect)
        if pos:
            self.rect.topleft = pos
        self.screen.blit(self.image, self.rect)
        
    def undraw_sprite(self):
        """remove the sprite from the screen surface.
        """
        if self.erasesurf:
            self.screen.blit(self.erasesurf, self.rect)
        else:
            self.screen.blit(self.backgr, self.rect, self.rect.inflate(2, 2))
        
    def get_name(self):
        return self.__name       

    def set_group_owner(self, group):
        """Set group that owns by this sprite"""
        self.group_owner = group        
    
    def set_use_current_background(self, bool=False):
        self.UseCurrentScreenAsBackground = bool
    
    def __eq__(self, x):
        return x == self.__name
    def __ne__(self, x):
        return x != self.__name
    
    def mouse_hover_enter(self,*args):
        """Called by the update method when a MOUSEMOTION event occurs and the 
        mouse is inside our rect.
        Derived gui widget classes should override this method."""
        self.hover_active = True
        
    def mouse_hover_leave(self,*args):
        """Called by the update method when a MOUSEMOTION event occurs and the 
        mouse is outside our rect.
        Derived gui widget classes should override this method."""
        self.hover_active = False
        
    def connect_callback(self, callback=None, event_type=None,  * args):
        """ Connect a callback function to a pygame event type or list of events, 
         with a optional arguments, the *args tuple.
         
         The event_type can be set to a pygame.event, this becomes the event
         to 'trigger' the callback. (see update method)"""
        self._callback = callback
        if type(event_type) is ListType:
            for t in event_type:
                self._event_type.append(t)
        else:
            self._event_type.append(event_type)
        self._args = args
                
    def disconnect_callback(self):
        """Disconnects the callback from the sprite.
        When you want to reconnect the callback you must reset it again with connect_callback"""
        c, e, a = self._callback, self._event_type, self._args
        self._args = [self]
        self._event_type = [] 
        self._callback = None
        return (c, e, a)

    def update(self, event=None):
        """ When there's a callback function set it will run the function, when
        the callback fuction return True the on_update method is called when it exist. 
        When you return the special value -1 the on_update isn't called and the
        value -1 is returnt to the caller. If the caller is the SPGroup it will stop
        looping the sprites and return -1 as the data value.
        This is used by some SPWidget.widgets to signal the eventloop to stop looping
        the widgets in the actives group.
        
         It is best to loop the event queue and look for a event, then call the group
         refresh method with the event, which updates  all the sprites as well as it 
         calls this method.
         It returns the return values of resp. on_update and the callback, or None 
        """ 
        cb, ou = None, None
        if event:
            if hasattr(self, '_scroll_focus') and not self._scroll_focus:
                return
            mouserect = pygame.mouse.get_pos() + (0, 0)
            if not event.type == MOUSEMOTION:
                if self._callback and event.type in self._event_type\
                    or hasattr(self, '_IamWidget') and (self._DragEnabled or self._DropEnabled):
                    # self.rect is the rect from the sub class
                    if event.type in (MOUSEBUTTONDOWN, MOUSEBUTTONUP) and self.rect.contains(mouserect) or\
                                    event.type == KEYDOWN or\
                                    event.type == USEREVENT + 1:
                        # First we do some widget focus stuff
                        if hasattr(self, '_IamWidget'):
                            if event.type == MOUSEBUTTONDOWN and self._CanHaveInputFocus:
                                SPGroup._WhoHasInputFocus = self
                            if event.type == KEYDOWN and SPGroup._WhoHasInputFocus is not self:
                                return
                            # here we check SPWidget objects for possible drag/drop events
                            if self._DragEnabled:
                                self.rect.contains(mouserect)
                                if event.type == MOUSEBUTTONDOWN and self.rect.contains(mouserect):
                                    self._logger.debug("start drag event")
                                    self._org_pos = self.rect.topleft
                                    
                                    self._prev_mouse_pos = pygame.Rect(mouserect).inflate(self.MinimalDragTreshold, self.MinimalDragTreshold)
                                    self.start_drag_event(self, (mouserect[0], mouserect[1]))
                                    return
                                elif event.type == MOUSEBUTTONUP:
                                    if type(self._prev_mouse_pos) is pygame.Rect:
                                        self._logger.debug("end drag event")
                                        self.end_drag_event(self, (mouserect[0], mouserect[1]))
                                        if not self._prev_mouse_pos.contains(mouserect):
                                            self._prev_mouse_pos =  None
                                            SPGroup._WhoWasDropped = self
                                            return -2# signal to caller Group
                                        elif self._callback and self.rect.contains(mouserect):
                                            self._prev_mouse_pos = None 
                                    else:
                                        return

                        # we check if we should call all possible matches
                        if event.type != USEREVENT+1 and self._callback:
                            if not (self.group_owner and self.group_owner.get_onematch()):
                                try:
                                    #print "--------call cbf-----------", event
                                    cb = apply(self._callback, (self, event, self._args))
                                except StandardError, info:
                                    self._logger.exception("Callback function %s failed" % self._callback)
                                    self._logger.error(info)
                                    raise MyError, info
                            else:
                                # we do not need to check existence of self.group_owner because we get
                                # here only if it exists.
                                if not self.group_owner.get_havematch():
                                    self.group_owner.set_havematch(True)
                                    try:
                                        cb = apply(self._callback, (self, event, self._args))
                                    except StandardError, info:
                                        self._logger.exception("Callback function %s failed" % self._callback)
                                        self._logger.error(info)
                                        raise Exception
                        if hasattr(self, '_IamWidget'):
                            pygame.time.wait(BUTTON_FEEDBACK_TIME)
                            if self.mouse_hover_leave_action and not self.ImSelected:
                                self.hover_active = False
                                self.image = self._but
                                self.erase_sprite()
                                self.display_sprite()
            elif event.type is MOUSEMOTION:
                collide = self.rect.collidepoint(pygame.mouse.get_pos())
                if collide and not self.hover_active:
                    self.mouse_hover_enter()
                elif self.hover_active and not collide:
                    self.mouse_hover_leave()            
        if cb == -1:
            return -1
        if cb:
            apply(self.on_update, self._args)
        else:
            ou = apply(self.on_update, self._args)
        return cb or ou
    
    def on_update(self,  *args):
        """ Always called by the group 'update' and 'refresh' methods and will
        move the sprite when a movement is set, you can override this to set
        your own 'update' stuff.
        
        This method just calls the 'next' method on the iterator returnt by
        the 'set_movement' method if it is set. Nothing more.
        When you override this method you must call 'next' yourself on the
        iterator returnt by 'set_movement'. But be aware that if you don't
        override this method you shouldn't call 'next' on the iterator as it
        would be called in here also.
        You can also override this for drag and drop animations."""
        if self._moveit_iter:
            try:
                self._moveit_iter.next()
            except StopIteration:
                self._moveit_iter = None
                self.stop_movement(now=1)
            except:
                self._logger.exception("Error while calling on_update. Killing sprite.")
                self.dokill = 1
                self.stop_movement(now=1)
    
    def set_movement(self, start=(0, 0), end=(0, 0), step=1, retval=0, loop=1, dokill=0):
        """ Define the movement of the object, it returns a generator.
        
         start = tuple x,y
         end = tuple x,y
         vector = tuple offset x, offset y
         retval = can be anything and is used as return value when the end of the 
         movement is reached.
         loop = times to loop before we signal retval, -1 means forever.
         dokill = kill the sprite by calling the kill() method when the movement is done.
        """
        self._stop_moveit = 0# flag used to stop a movement
        self._start, self._end, self._step, self._retval, self._loop = \
            start, end, step, retval, loop
        self.rect.topleft = self._start# place it in the start position
        #print "sprite moved to",self._start
        self.dokill = dokill 
        # also a class attribute so that on_update can call it.
        self._moveit_iter = self._moveit()
        return self._moveit_iter
        
    def _moveit(self):
        """ Move this object according to the values in movement.
        
         The set_movement method should be called before this one.
         Returns -1 when the movement continues, and the given return value when
         it stops. (the retval argument from set_movement)."""
        if self._stop_moveit:#test if the flag is set, see def stop_moveit
            yield self._retval
        
        x0, y0 = self._start
        x1, y1 = self._end
        
        dx = x1 - x0
        dy = y1 - y0
        dist = math.sqrt((dx * dx) + (dy * dy))
        nhops = int(dist)
        for i in range(1, 1 + nhops, self._step):
            if self._stop_moveit:#test if the flag is set, see def stop_moveit
                yield self._retval
            x, y = x0 + dx * i / nhops, y0 + dy * i / nhops
            # Remember that self.rect is a mandatory member of any SPSprite class
            self.rect.topleft = (x, y)
            #print x,y,self.rect
            yield -1
        if self._loop == -1: # we loop forever
            self._moveit() # recursive call 
        # when we are here, we loop a finite amount of times
        self._loop -= 1
        if self._loop > 0:
            self._moveit()
        # when we are here the motion is ended.
        self.stop_movement(1)
        yield self._retval
        
    def stop_movement(self, now=None):
        """This stops any running movement, when 'now' is set it will terminate
        
        the movement by changing the return value from 'moveit', else we change the
        loop attribute. This way the movement stops when the current run is done."""
        if now:
            self._stop_moveit = 1
            self._loop = 0# to be sure
        else:
            self._loop = 0
        if self.dokill:
            self.erase_sprite()
            self.kill()
            
    def moveto(self, pos, hide=True):
        """ Move the sprite to a new position.
        
        This will erase the sprite on it's old position and redraw it on the new
        position.
        When @hide is True the sprite is only moved but not shown"""
        if hide:
            self.rect.topleft = pos
        else:
            self.erase_sprite()
            self.display_sprite(pos)
    
    def display_sprite(self, pos=None):
        """ Display a sprite without the need to call group methods.
        
        Usefull for just displaying this sprite, nothing more.
        It takes an optional argument 'pos' which will place the sprite to
        that location."""
        if hasattr(self, '_scroll_focus') and not self._scroll_focus:
            return
        if self.UseCurrentScreenAsBackground:
            #self.backgr = pygame.display.get_surface()
            self.erasesurf = pygame.Surface(self.image.get_size())
            self.erasesurf.blit(pygame.display.get_surface(), (0, 0), self.rect)
        if pos:
            self.rect.topleft = pos
        r = self.screen.blit(self.image, self.rect)
        #print "SPSpriteUtils.display_sprite", self.rect
        pygame.display.update(r)

    def erase_sprite(self):
        """ Erase a sprite without the need to call group methods.
        
        Usefull for just erasing this sprite, nothing more."""
        #print "self",self,"erase rect",self.rect
        #r = self.screen.blit(self.backgr, self.rect.inflate(6, 4), self.rect.inflate(6, 4))
        #r = self.screen.blit(self.backgr, self.rect, self.rect)
        if self.erasesurf:
            r = self.screen.blit(self.erasesurf, self.rect)
        else:
            r = self.screen.blit(self.backgr, self.rect, self.rect.inflate(2, 2))
        pygame.display.update(r)
    
    def refresh_sprite(self):
        """Erase and display sprite in one call"""
        if hasattr(self, '_scroll_focus') and not self._scroll_focus:
            return
        if self.erasesurf:
            r = self.screen.blit(self.erasesurf, self.rect)
        else:
            r = self.screen.blit(self.backgr, self.rect, self.rect.inflate(2, 2))
        if self.UseCurrentScreenAsBackground:
            #self.backgr = pygame.display.get_surface()
            self.erasesurf = pygame.Surface(self.image.get_size())
            self.erasesurf.blit(pygame.display.get_surface(), (0, 0), self.rect)
        rr = self.screen.blit(self.image, self.rect)
        pygame.display.update([r,rr])
        
    def remove_sprite(self):
        """ Remove this sprite object from all the groups it belongs.
        
          as well as remove it from the screen. This calls pygame.display.update
          The sprite is returnt so that you can keep a reference.
        """
        #self.screen and self.backgr are set by _Sprite_setup
        self.kill()# removes this object from any group it belongs
        self.erase_sprite()
        return self
       
    def get_sprite_height(self):
        """ Returns the height of the surface used in this sprite"""
        return self.image.get_height()
    def get_sprite_width(self):
        """ Returns the width of the surface used in this sprite"""
        return self.image.get_width()
    def get_sprite_rect(self):
        """ Returns the rect of the sprite"""
        return self.rect
    def get_sprite_pos(self):
        """ Returns the position of the sprite"""
        return self.rect.topleft

class SPGroup(pygame.sprite.RenderUpdates):
    _WhoHasInputFocus = None
    _WhoWasDropped = None
    """ Group  wich extends the standard RenderUpdates group.
    
    This is meant to use together with the SPSprite class.
    You MUST call the init function BEFORE you use anything in this module.
    
    Look at the examples that came with this module for possible usage.
    If you want to grok this magic, look at the pygame docs (sprite classes).
    The best you can do is just use the magic from the example ;-)
     """
    def __init__(self, scr, bck):
        """__init__(scr,bck)
           scr = reference to the display screen.
           bck = reference to a background screen."""
        self._logger = logging.getLogger("childsplay.SPSpriteUtils_lgpl.SPGroup")
        self.DEBUG = 0
        pygame.sprite.RenderUpdates.__init__(self)
        self.scr = scr
        self.bck = bck
        self.__onematch = False
        self.__havematch = False 
        self.__dropables = []
        
    def set_input_focus(self, widget):
        SPGroup._WhoHasInputFocus = widget
    
    def who_has_input_focus(self):
        return SPGroup._WhoHasInputFocus
    
    def register_dropable(self, widget):
        self.__dropables.append(widget)

    def refresh(self,  * args):
        """ Clear the sprites, calls the update method on the sprites,
        redraw the sprites and update the display.
        It returns the return values of any on_update or callback function.
        Look at the update method for a example of the stuff returnt."""
        self.clear(self.scr, self.bck)
        if args:
            v = self.update(args[0])
        else:
            v = self.update()
        if self.DEBUG: print >> sys.stderr, self.__class__, "returnt from self.update", v
        rects = self.draw(self.scr)    
        pygame.display.update(rects)
        return v
    
    def redraw(self):
        """ Clear the sprites, redraw the sprites that have a their _scroll_focus attribute set and those who 
        haven't a scroll_focus attribute (regular sprites).
        """
        for s in self.sprites():
            if hasattr(s, '_scroll_focus') and not s._scroll_focus:
                return
            s.erase_sprite()
            s.display_sprite()
    
    def get_stack_ids(self):
        return self.sprites()

    def get_sprites(self):
        """ get_sprites, returns a list of the sprites currently in the group.
        It's almost the same as group.sprites but this will always return a list,
        group.sprites can return in the future an iterator.
        Anyway, use this if you want a real list of your sprites."""
        return [x for x in self.sprites()]

    def add(self, *sprites):
        """add a sprite to group

        add(...) reuses pygame.sprite.RenderUpdates.add to add a sprite or
        sequence of sprites to a group.
        Before actual adding the sprite to the group, the linkage between the
        sprite and the group is set"""
        for sp in sprites:
            if isinstance(sp, SPSprite):
                sp.set_group_owner(self)
        pygame.sprite.RenderUpdates.add(self, *sprites)
    
    def set_onematch(self, onematch=False):
        """Manipulate the behaviour of the sprites to an event.
        
        When @onematch is true then the calling of callbacks stops after one
        matching event. For example when you have multiple sprites on top of each other
        and the user clicks on the top sprite, all the sprites below the top one 
        reacts to the event as if they were also clicked.
        By setting one_match only one sprite reacts and the rest is ignored."""
        #self._logger.debug("onematch is set to %s", onematch)
        self.__onematch = onematch

    def get_onematch(self):
        #self._logger.debug("onematch is %s", self.__onematch)
        return self.__onematch

    def set_havematch(self, havematch=False):
        """Manipulates the flag which reflects whether the group of the sprites
        has already handles an event
        
        If __onematch attribute of the group is set to true and a sprite within
        the group triggers an event, it sets __havematch in true as so other
        sprites in the group will not handle this event
        The flag is used to re-set to false as soon as all sprites within the
        group are updated with the event"""
        #self._logger.debug("havematch is set to %s", havematch)
        self.__havematch = havematch

    def get_havematch(self):
        #self._logger.debug("havematch is %s", self.__havematch)
        return self.__havematch

    def purge(self):
        """Will remove all sprites from this group and erase them from the screen."""
        for s in self.sprites():
            s.erase_sprite()
        self.empty()

    def update(self, *args):
        """update(...), this overrides the pygame.sprite.group.update
           call update for all member sprites

           calls the update method for all sprites in the group.
           Passes all arguments on to the Sprite update and callback function.
           It returns all the return values of these functions in a list.
           
           Example: [(sprite name,return val),(sprite name,return val)]
           sprite name is sprite.__class__, so you can change it to make the sprite name unique.
           return val is the val returnt by the SPSprite.update method and it's the value from the callback
           function if it exist otherwise it's from the on_update.
           If your sprite returns -1 the loop will break and no other sprites will be called.
           The return value -2 is used by DnD objects"""
        l = []
        l_append = l.append
        if args:
            a = apply
            for s in self.spritedict.keys():
                v = a(s.update, args)
                if v:
                    #self._logger.debug("returnt from SPSprite update %s" % v)
                    l_append((s, v))
                    if v == -1:
                        pygame.event.clear()
                        self._logger.debug("Got a -1 value breaking out of the update loop")
                        break
                    elif v == -2 and SPGroup._WhoWasDropped:
                        obj = SPGroup._WhoWasDropped
                        index = obj.rect.collidelist(self.__dropables)
                        if index != -1:
                            if self.__dropables[index].drop_event(obj):
                                obj._DnDreset()
                            break
                        else:
                            SPGroup._WhoWasDropped = None
                            obj._DnDreset()
                        
        else:
            for s in self.spritedict.keys():
                s.update()
        # reset any match 
        # do not use set_havematch here due to overhead - update() is invoked too often
        self.__havematch = False
        return l
        
class CPStackGroup:
    """ Group which mimics the pygame.sprite groups, but keeps the sprites
    it contains on a stack so that you have control over the way the sprites are handled.
    
    Besides typical stack like stuff like pop and push this group keeps a reference
    to each sprite that gets added, which is also returned when the sprite is added.
    Use this group when you want to manage the way and moment each sprite is updated.
    The way the sprites are cleared is by getting a run-time background rect of the area
    of the sprite just before it gets drawn.
    The stack makes also possible to raise or lower sprites.
    See the actual methods on usages.
    Be aware that this group sprite updating methods involves a lot of overhead, so use
    it only if you need some of the features it provides.
    
    You MUST call the init function BEFORE you use anything in this module.
    """
    _spritegroup = 1 #dummy val to identify groups
    
    def __init__(self, scr, bck):
        """__init__(scr,bck)
           scr = reference to the display screen.
           bck = reference to a background screen.
        """
        self.DEBUG = 1
        self.stack = []# becomes [(ID,sprite),(ID,sprite),...]
        self.ID = 0 # becomes self.ID += 1 : str(self.ID)
        self.scr = scr
        self.bck = bck

    def add(self, sprite):
        """add(sprite)
           add sprite to the group and push it/them on the stack."""
        try: 
            len(sprite) #see if its a sequence
        except (TypeError, AttributeError):
            id = self.push(sprite)
            sprite.add_internal(self)
            return id
        else:
            t = ()
            for s in sprite:
                t = t + self.push(s)
                s.add_internal(self)
            return t

    def get_sprite(self, id):
        """get(id)
           Get a sprite from the stack.
           
           This returns a sprite but does NOT remove it.
           """
        found = self._find_in_stack(id)
        if found != -1:
            s = self.stack[found][1]
            return s
            
    
    def push(self, sprite):
        """push(sprite)
           Push a sprite on the stack."""
        self.ID += 1
        id = str(self.ID)
        self.stack.append((id, sprite))
        return id
    
    def pop(self, index=-1):
        """pop(index=-1)
           Pop a sprite from the stack, remove and return the last one added.
           
           Optional you can give a index which item to pop.
           It defaults to -1, the last item."""
        try:
            s = self.stack.pop(index)[1]
        except IndexError:
            return None
        else:
            r = self.scr.blit(self.bck, s.rect, s.rect)
            #print "pop clear",s.rect,"rect",r
            pygame.display.update(r)
        return s
        
                     
    def _find_in_stack(self, sprite):
        # search the stack for a sprite, sprite can be a sprite object or a ID string
        index = 0
        for item in self.stack:
            if sprite in item:
                return index
            else:
                index += 1
        return -1# not found      
        
    def add_internal(self, sprite):
        self.push(sprite)


    def remove(self, sprite):
        """remove(sprite)
           remove sprite from the stack and group

           Remove a sprite or sequence of sprites from the stack and group."""
        try:
            len(sprite) #see if its a sequence
        except (TypeError, AttributeError):
            found = self._find_in_stack(sprite)#returns a index
            if self.DEBUG == 1: print "found", found
            if found != -1:
                self.remove_internal(found)
                sprite.remove_internal(self)
        else:
            for s in sprite:
                found = self._find_in_stack(s)
                if found != -1:
                    self.remove_internal(found)
                    sprite.remove_internal(self)
                    
    def remove_internal(self, index):
        found = self._find_in_stack(index)
        if found != -1:
            self.pop(found)
            

    def draw(self,  * args):
        """draw(surface)
           draw all sprites onto the surface

           Draws all the sprites onto the given surface. It
           creates a list of rectangles, which which are passed
           to pygame.display.update()
           Sprites are first erased and then drawn "FILD",
           First In Last Drawn."""
        bck = self.bck
        scr_blit = self.scr.blit
        
        stack = self.stack[:]
        rects = []
        rects_append = rects.append
        for id, sprite in stack:
            # first we erase the sprite by blitting the old backgr
            rects_append(scr_blit(bck, sprite.rect, sprite.rect))
            if sprite.update( * args):# callback function returns true/false
                break# then we considered to stop 
            # now we draw them 
            rects_append(scr_blit(sprite.image, sprite.rect))
        pygame.display.update(rects)
        
            
    def refresh(self, event):
        """ refresh(*args)
             Calls the update and draw methods on all the sprites belonging to this group.
        """
        #self.update(*args)
        self.draw(event)
        #print "Stack",self.stack

    def update(self, * args):
        """update(...)
           call update for all member sprites

           calls the update method for all sprites in the group.
           passes all arguments are to the Sprite update function."""
        if args:
            a = apply
            stack = self.stack
            for i, s in stack:
                a(s.update, args)
        else:
            for i, s in stack:
                s.update()

    def get_stack_ids(self):
        """ get_stack_ids()
            Get a list with the current IDs from the stack.
            
            Use this to get a overview of the contents of the stack but use pop, push etc
            to handle the stack.
            """
        ids = []
        ids_append = ids.append
        for i in self.stack:
            ids_append(i[0])
        return ids
    
    def up(self, id):
        """ up(ID)
            Raise the sprite to the top of the stack.
            """
        index = self._find_in_stack(id)
        if index != -1:
            self.stack.append(self.stack.pop(index))
                
    def down(self):
        """ down() 
            Get the sprite on top of the stack and put it on the bottom of the stack.
            """
        self.stack.insert(0, self.stack.pop())

class MySprite(SPSprite):
    """Container to turn a surface into a SPSprite object"""
    def __init__(self, value, name=None, pos=(0, 0)):
        """Turn an image into a schoolsplay sprite.
        @value can be a SDL surface or a string.
        Only use this if you need a simple sprite.
        For anything else let your sprite derive from SPSprite. 
        See the SPSprite api documentation for info about this class.
        value must be an pygame surface or a path to a supported image file.
        name is optional and can be used to identify this sprite.
        See also the SPSprite class."""
        self._logger = logging.getLogger("childsplay.SPSpriteUtils_lgpl.MySprite")
        if type(value) is pygame.Surface:
            self.image = value
        elif type(value) in (types.StringType, types.UnicodeType):
            try:
                self.image = load_image(value)
            except Exception, info:
                self._logger.error("failled to load image: %s" % value)
                raise MyError, info
        self.rect = self.image.get_rect()
        SPSprite.__init__(self, self.image, name=name)
        self.moveto(pos)
        
class SPButton(SPSprite):
    """ Turn an image into a button like object which reacts to mouseclicks.
    It will connect the 'MOUSEBUTTONDOWN' event to the callback function named
    'callback'.
    This callback function returns the data given in the class constructor when
    called from the eventloop.
    DEPRECATED; Use Button
    """
    def __init__(self, img, pos, data):
        """@img is an SDL image object.
        @pos is a tuple with a x,y grid numbers.
        @data is returnt by the callback function when the MOUSEBUTTONDOWN occurs
        """
        self.image = img
        self.rect  = self.image.get_rect()
        SPSprite.__init__(self, self.image)
        self.rect.move_ip(pos)
        self.data = data
        self.connect_callback(self.callback, event_type=MOUSEBUTTONDOWN)
    
    def callback(self,  * args):
        return self.data   



def set_big_mouse_cursor(file=None, mask=None):
    """This will replace the default cursor with a bigger one"""
    thickarrow_strings = (#sized 24x24
        "XX                      ",
        "XXX                     ",
        "XXXX                    ",
        "XX.XX                   ",
        "XX..XX                  ",
        "XX...XX                 ",
        "XX....XX                ",
        "XX.....XX               ",
        "XX......XX              ",
        "XX.......XX             ",
        "XX........XX            ",
        "XX........XXX           ",
        "XX......XXXXX           ",
        "XX.XXX..XX              ",
        "XXXX XX..XX             ",
        "XX   XX..XX             ",
        "     XX..XX             ",
        "      XX..XX            ",
        "      XX..XX            ",
        "       XXXX             ",
        "       XX               ",
        "                        ",
        "                        ",
        "                        ")
    if file:
        cursordata = pygame.cursors.load_xbm(file, mask)
    else:
        data, mask = pygame.cursors.compile(thickarrow_strings, black='X', white='.', xor='o')
        hot = (0, 0)
        size = (24, 24)
        cursordata = (size, hot, data, mask)
    pygame.mouse.set_cursor( * cursordata)
    
def set_no_mouse_cursor():
    """This will replace the default cursor with one that's empty.
    This is used to hide the cursor"""
    thickarrow_strings = ( #sized 8x8
        "        ",
        "        ",
        "        ",
        "        ",
        "        ",
        "        ",
        "        ",
        "        ")

    cursor = pygame.cursors.compile(thickarrow_strings, black=".", white="X")
    pygame.mouse.set_cursor((8,8), (5,0), *cursor)
    
    



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPMenu.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Menu for the activities
# Special activity to provide a menu for maincore to load other activties

# We construct a hash with lists of tuples with xml items
# menu = {(<submenu icon name>,<subicon pos>): (<act icon>, <act pos>, <act name>)}


# In Activity we turn them into buttons.
# Activity buttons return a module name and submenu buttons return other buttons.

import logging
import types
try:
    from xml.etree.ElementTree import ElementTree
except ImportError:
    # try the python2.4 way
    from elementtree.ElementTree import ElementTree

import pygame
from pygame.constants import *

import utils
from SPConstants import *
from SPHelpText import ActivityMenuText
from SPWidgets import Button, ImgButton, TransImgButton, ImgTextButton
from SPSpriteUtils import SPInit, SPSprite
import SPDataManager
import Version

module_logger = logging.getLogger("childsplay.SPMenu")



class ParseMenu:
    def __init__(self, xml, complete=False):
        self.logger = logging.getLogger("childsplay.SPMenu.ParseMenu")
        self.tree = ElementTree()
        self.tree.parse(xml)
        root = self.tree.getroot()
        rootdefault = root.get('defaultsubmenu')
        menuhash = {'menudefault':rootdefault}
        
        submenus = self.tree.findall('submenu')
        
        for sub in submenus:
            menlist = []
            subicon = sub.get('file')
            subhicon = sub.get('hoverfile')
            # when theres no position we assume the images together will be 800 width.
            # and we add postions based on the width of the images.
            try:
                subpos = tuple([int(x) for x in sub.find('position').text.split(',')])
            except AttributeError:
                # we will calculate the position based on image size
                subpos = (-1, -1)
            for item in sub.findall('menuitem'):
                menact = item.find('activity').text
                if item.get('enable') != 'True':
                    self.logger.debug("activity %s removed from menu, enable != True" % menact)
                    if complete:
                        enabled = False
                    else:
                        continue
                else:
                    self.logger.debug("activity %s added to the menu" % menact)
                    enabled = True
                menicon = item.get('file')
                menhicon = item.get('hoverfile')
                mentext = item.get('text')
                if mentext == 'True':
                    mentext = True
                else:
                    mentext = False
                try:
                    menpos = tuple([int(x) for x in item.find('position').text.split(',')])
                except AttributeError:
                    # we will calculate the position based on image size
                    menpos = (-1, -1)
                menlist.append((menicon, menhicon, menpos, menact, mentext, enabled))
            menuhash[((subicon, subhicon), subpos)] = menlist
        self.menuhash = {}
        for k, v in menuhash.items():
            if k == 'menudefault':
                self.menuroot = v
            else:
                self.menuhash[k] = v
                        
    def get_menu(self):
        return self.menuhash
    def get_menudefault(self):
        return self.menuroot
      
class Menu:
    def __init__(self, iconpath, menu, cbf, theme, lang, default, removeables=None):
        self.logger = logging.getLogger("childsplay.SPMenu.Menu")
        lang = lang[:2]
        buttons = utils.OrderedDict()
        acttext = ActivityMenuText()
        self.buttonslist = []
        self.defaultbuttonslist = []
        default_iconpath = iconpath
        self.removeables = removeables# list of button names that should be removed
        # menu = {(<submenu icon name>,<subicon pos>): [(<act icon>, <act pos>, <act name>)]}
        # the hash key 'rootmenu' is removed from menu
        # here we construct the menu buttons
        #
        # It's possible to have localized menu act buttons.
        # First we look if there's a subdir in /lib/SPData/<theme>/menuicons named
        # to the current locale, eg /lib/SPData/<theme>/menuicons/nl for dutch.
        # If so we use those icons, if not we use the default ones in 
        # /lib/SPData/<theme>/menuicons/
        # If there's a localized subdir we use it, missing icons are replaced with the default ones.
        if os.path.exists(os.path.join(iconpath, lang)):
            iconpath = os.path.join(iconpath, lang)
        max_x = 790
        x_padding =50
        for k, v in menu.items():
            #self.logger.debug("building menu item: %s,%s" % (k, v))
            x, y = 50, 110 # start coords when theres no position given for the menu buts  
            for t in v:
                if os.path.splitext(t[0])[0].rstrip('.icon') in self.removeables:
                    continue
                p = os.path.join(iconpath, t[0])
                if not os.path.exists(p):
                    p = os.path.join(default_iconpath, t[0])
                if t[1]:
                    hp = os.path.join(iconpath, t[1])
                    if not os.path.exists(hp):
                        hp = os.path.join(default_iconpath, t[1])
                else:
                    hp = None
                if t[2][0] == -1:
                    pos = (x, y)
                else:
                    pos = t[2]
                print t
                if t[4]:
                    try:
                        txt = getattr(acttext, t[3])
                    except AttributeError:
                        self.logger.warning("No text attribute found in SPHelpText for %s" % t[3])
                        b = ImgButton(p, pos, name=t[3])
                    else:
                        b = ImgTextButton(p, _(txt) ,pos, padding=6, fsize=14, name=t[3])
                else:    
                    if theme['menubuttons'] == 'transparent':
                        b = TransImgButton(p, hp, pos, name=t[3])
                    else:
                        b = ImgButton(p, pos, name=t[3])
                b.connect_callback(cbf, MOUSEBUTTONUP, t[3])
                # we set the name of the activity to be used to identify the button later
                b.name = os.path.splitext(t[0])[0].rstrip('.icon')
                x += b.get_sprite_width() + x_padding
                if x > max_x - b.get_sprite_width():
                    x = 50
                    y += 140
                if buttons.has_key(k):
                    buttons[k].append(b)
                else:
                    buttons[k] = [b]
        # here we construct the category buttons
        # first we must determine which button list belongs to the left 
        x , y = 0, 532 # start coords when theres no position given for the submenu buts  
        for k, v in buttons.items():
            p0 = os.path.join(iconpath, 'submenu', k[0][0])
            p1 = os.path.join(iconpath, 'submenu', k[0][1])
            if k[1][0] == -1:
                pos = (x, y)
            else:
                pos = k[1]
            b = TransImgButton(p0, p1, pos, padding=0, values=v, name=os.path.splitext(k[0][0])[0])
            if k[1][0] == -1:
                x += b.get_sprite_width()
            if k[0][0] == default:
                self.defaultbuttonslist = v
                self.defaultbutton = b
            b.set_use_current_background(True)
            b.connect_callback(cbf, MOUSEBUTTONUP, v)
            self.buttonslist.append(b)
            
    def _set_default_buttons(self, blist):
        self.defaultbuttonslist = blist
    def get_default_buttons(self):
        return (self.defaultbutton, self.defaultbuttonslist)
    def get_buttons(self):
        return self.buttonslist
    
class Activity:
    """  This provides the activity menu internally used by MainCore.
    
    It's much like an 'regular' activity but it is called differently
    """
    def __init__(self, copmode=False, showpersonalquiz=False, showlocalquiz=False):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        This SPGoodies differs from the regular SPGoodies the activities get."""
        self.logger = logging.getLogger("childsplay.SPMenu.Activity")
        self.logger.info("Activity started")
        self.displayed_buttons = []
        self.displayed_bottom_buttons = []
        self.selected_button = None
        self.COPmode = copmode
        self.removeables = []
        if not showpersonalquiz:
            self.removeables.append('quiz_personal')
        if not showlocalquiz:
            self.removeables.append('quiz_regional')   
    
    def _setup(self, SPGoodies, dm, bottombar):
        self.SPG = SPGoodies
        self.dm = dm
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        
        self.theme_dir = os.path.join(self.SPG.get_libdir_path(), 'SPData', 'themes', self.SPG.get_theme())
        self.theme_rc = self.SPG.get_theme_rc()
        self.actives = SPInit(self.screen,self.backgr)        
        self.actives.set_onematch(True)
                
        img = utils.load_image(os.path.join(self.theme_dir, bottombar))
        self.bottom_bar = SPSprite(img)
        if self.COPmode:# used for the btp controlpanel
            self.bottom_bar.moveto((0, 900))
        else:
            self.bottom_bar.moveto((0, 534))
        
    def _parse_menu(self, theme_dir, xmlname):
        p = os.path.join(theme_dir, xmlname)
        try:
            Pm = ParseMenu(p)
        except Exception, info:
            self.logger.exception("Error while parsing menu xml file: %s" % p)
            raise utils.MyError, info
        self.menu = Pm.get_menu()
        self.menudefault = Pm.get_menudefault()
    
    def _build_menu(self, theme_dir, theme_rc, lang):
        p = os.path.join(theme_dir, 'menuicons')
        try:
            self.Mn = Menu(p, self.menu, self.menu_callback, theme_rc, lang, self.menudefault, \
                           removeables=self.removeables)
        except Exception, info:
            self.logger.exception("Error while constructing the menu buttons")
            raise utils.MyError, info
        
    def _remove_buttons(self, buttons):
        #self.logger.debug("_remove_buttons called with:%s" % buttons)
        if not self.actives:
            return
        for b in buttons:
            b.erase_sprite()
        self.actives.remove(buttons)
    
    def _display_buttons(self, menubuttons):
        if len(menubuttons) == 0:
            self.logger.error("No buttons found to display")
            raise utils.MyError, "No buttons found to display, check your install"
            return
        if menubuttons == self.displayed_bottom_buttons:
            refresh = False
        else:
            refresh = True
        for b in menubuttons:
            if refresh:
                b.mouse_hover_leave()
            b.display_sprite()
        self.actives.add(menubuttons)
    
    def menu_callback(self, sprite, event, data):
        #self.logger.debug('menu_callback called with sprite %s, event %s and data %s' % (sprite, event, data))
        pygame.time.wait(200)
        if type(data[0]) not in types.StringTypes:
            self.logger.debug("menu cbf data is object list")
            if self.selected_button:
                self.selected_button.unselect()
            self.selected_button = sprite
            sprite.select()
            # clear the toplevel buttons
            self._remove_buttons(self.displayed_buttons)
            menubuttons = data[0]
            self.displayed_buttons = menubuttons
            self._display_buttons(menubuttons)
        else:
            self.logger.debug("menu cbf argument is module name")
            self.Mn._set_default_buttons(self.displayed_buttons)
            self._remove_buttons(self.displayed_buttons)
            self._remove_buttons(self.displayed_bottom_buttons)
            self.bottom_bar.erase_sprite()
            self.SPG._menu_activity_userchoice(data[0])
            return
    
    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.redraw()

    def get_helptitle(self):
        """Mandatory method"""
        return "BrainTrainerPlus"
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "menu"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("This is the menu used to select an activity."),
        _("Select an activity and click on the button, the activity will be started."),
        _("Use the 'sub menu' buttons for the different activity menus"),
        " ",
        "Developers: Stas Zytkiewicz, Chris van Bael",
        "Artwork and support: Formatics.nl, BraintrainerPlus.com, Wysiwich.be",
        " ",
        "This program is free software; you can redistribute it and/or modify it under the terms of version 3 of the GNU General Public License as published by the Free Software Foundation.",
        "A copy of this license should be included in the file GPL-3.",
        " ",
        "We tried to make sure we don't include artwork or sounds that might be copyrighted by someone else but if you are the copyright holder of something inside Childsplay please let us know at info@childsplay.mobi and we remove it."
        " ",
        "Version: %s" % Version.version
        ]
        if self.theme_rc['theme'] == 'braintrainer':
            text = [_("A collection of braintrain games for seniors."),
                    _("All the games have there own help page, just start a game and hit the stars on top of the screen to select another difficulty."),
                    _("Use the menu below to choose between the different game categories"),
                    " ",
                "Credits:",
                "Based on the Open Source Game framework Seniorplay (www.schoolsplay.org)",
                "The source code for this program is free software; you can redistribute it and/or modify it under the terms of version 3 of the GNU General Public License as published by the Free Software Foundation.",
                "A copy of this license should be included in the file GPL-3.", 
                " ",
                "Artwork and game content is property of QiosQ.com and is excluded from the GPL-3 license."
                "Artwork, Content and Support by QiosQ", 
                "Version: %s" % Version.version
                    ]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty string"""
        return ""
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        return ""
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels" % number-of-levels)"""
        return ""    

    def stop_timer(self):
        pass
    
    def start(self):
        """Mandatory method.
        This will start first level"""
        self.logger.debug("starting")
        self.scoredisplay = self.SPG.get_scoredisplay()
                
    def pre_level(self, level):
        """Mandatory method"""
        pass    
    def next_level(self, level):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("next_level")
        self.bottom_bar.display_sprite()
        menubuttons = self.Mn.get_buttons()
        #self.SPG.tellcore_enable_dice(1)
        self.SPG.tellcore_enable_dice(False)
        self.SPG.tellcore_hide_level_indicator()
        self.scoredisplay.clear_score()
        
        if self.displayed_buttons:
            # we are not in the start menu so we keep the stuff we had
            self._display_buttons(self.displayed_buttons)
            self._display_buttons(self.displayed_bottom_buttons)
            self.selected_button.select()
        else:
            self.displayed_bottom_buttons = menubuttons
            self._display_buttons(menubuttons)
            # display the default buttons
            but, menubuttons = self.Mn.get_default_buttons()
            self.displayed_buttons = menubuttons
            self._display_buttons(menubuttons)
            but.select()
            self.selected_button = but
        # we always return true as we don't have real levels        
        return True
        
    def loop(self, events):
        """Mandatory method"""
        for event in events:
            self.actives.update(event)
        return 
    
    



# -*- coding: utf-8 -*-

# Copyright (c) 2011 Rene Dohmen acidjunk@gmail.com
#
#           filewalker.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation. A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


####### IMPORTANT READ THIS ####################################################
# This is a script to collect info for the photoalbum                          #
################################################################################

import os, sys, glob

xml_header = """<?xml version="1.0" encoding="utf-8"?>
<album name="Museum in echt 2010" nofPicture="79">"""
xml_footer = """</album>
</xml>"""

xml = xml_header

dirname = "nl_Album_4"
os.chdir(dirname)
matches = glob.glob('*.jpg')
matches.sort()
for fileName in matches:
    #os.system("mv %s %s" % (fileName,fileName[:-4]))
    print "Found: %s" % fileName
    title=raw_input("\nEnter title:")
    text=raw_input("\nEnter text:")
    xmlPart="""\t<photo name="%s">
        <title>%s</title>
        <text>%s</text>
    </photo>""" % (fileName,title,text)
    xml = "%s\n%s" % (xml, xmlPart)
print xml
   







# -*- coding: utf-8 -*-

# Copyright (c) 2010-2011 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           ichanger.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.ichanger.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.ichanger")

# standard modules you probably need
import os,sys, random, glob

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils
import SPWidgets
from SPHelpText import Act_ichanger
# containers that can be used globally to store stuff
class Img:
    pass
class Snd:
    pass

class Card(SPSpriteUtils.SPSprite):
    def __init__(self,closedcard,opencard,pos, observer, name):
        """This class has it's own callback function and it will connect itself"""
        SPSpriteUtils.SPSprite.__init__(self,opencard)
        self.rect.move_ip(pos)
        self.name = name
        self.closedimage = closedcard
        self.openimage = opencard.convert_alpha()
        self.observer = observer
        self.IMchanged = False
        self.wrongs = 0
        self.changed_image = None
        self.org_image = opencard.convert_alpha()
        self.newpos = pos
        self.orgpos = pos
        # connect to the MOUSEBUTTONDOWN event, no need for passing data
        self.connect_callback(self.callback,MOUSEBUTTONDOWN)
    
    def reset(self):
        self.openimage = self.org_image
        self.moveto(self.orgpos)
    
    def restart(self):
        self.moveto(self.newpos)
        if not self.IMchanged:
            self.openimage = self.org_image
        else:
            self.openimage = self.changed_image
        
            
    def close(self):
        self.image = self.closedimage
        self.display_sprite()
               
    def open(self):
        self.image = self.openimage
        self.display_sprite()
        
    def change_image(self):
        if not self.changed_image:
            return
        self.openimage = self.changed_image
        self.IMchanged = True
        
    def set_changed_image(self, image):
        self.changed_image = image
        
    def set_new_position(self, pos):
        self.moveto(pos)
        self.newpos = pos
                
    def callback(self,sprite,event,*args):
        if self.IMchanged:
            #print "correct"
            pygame.draw.rect(self.image,ROSYBROWN,self.image.get_rect(), 12)
            self.display_sprite()
            self.observer(True)
        else:
            self.wrongs += 1
            #print "wrong"
            self.close()
            self.disconnect_callback()
            self.observer(False)

class TextBox(SPSpriteUtils.SPSprite):
    def __init__(self, surf, pos):
        """This class has it's own callback function and it will connect itself"""
        SPSpriteUtils.SPSprite.__init__(self,surf)
        ai = Act_ichanger()
        self.textsurfs = []
        for txt in (ai.box1, ai.box2, ai.box3):
            s = SPWidgets.render_textrect(_(txt), 18, TTF, surf, WHITE, \
                              None, justification=0, \
                              autofit=False, border=0, padding=24)
            self.textsurfs.append(s)
        self.moveto(pos)
        
    def set_text(self, i):
        self.image = self.textsurfs[i]
        self.display_sprite()
                
class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.ichanger.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.displayedscore = 0
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','IchangerData')
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'ichanger.rc'))
        self.rchash['theme'] = self.theme
        self.my_rchash = self.rchash[self.theme]
        self.logger.debug("found rc: %s" % self.rchash)
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
        
        imgdir = os.path.join(self.my_datadir,'images', self.theme)
        if not os.path.exists(imgdir):
            imgdir = os.path.join(self.my_datadir, 'images','childsplay')
        
        self.imagepaths_org = [f for f in glob.glob(os.path.join(imgdir, '*'))\
                            if os.path.basename(f) not in ('cardfront.png', 'cardback.png')]
        self.imagepaths = self.imagepaths_org[:]
        random.shuffle(self.imagepaths)
        self.opencard = utils.load_image(os.path.join(imgdir,'cardfront.png'))
        self.closedcard = utils.load_image(os.path.join(imgdir,'cardback.png'))
        
        boxsurf = utils.load_image(os.path.join(self.my_datadir,'box.png'))
        self.pausesnd = utils.load_music(os.path.join(self.CPdatadir,'pause_0_35.ogg'))
        self.good_sound=utils.load_music(os.path.join(self.CPdatadir,'good.ogg'))
        self.wrong_sound=utils.load_music(os.path.join(self.CPdatadir,'wrong.ogg'))
        p = os.path.join(self.CPdatadir,'good_%s.png' % self.lang)
        if not os.path.exists(p):
            p = os.path.join(self.CPdatadir,'thumbs.png')
        self.good_image = SPSpriteUtils.MySprite(utils.load_image(p))
        self.good_image.moveto((229,296))
        # Your top blit position, this depends on the menubar position
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        self.cardpositions = [(10, y), (260, y), (10, y+250), (260, y+250)]
        #self.start_pos = (600, 380 + y)
        self.but_pos = [(560, 420 + y), (700, 400+y)]
        boxpos = (510, y)
        self.box = TextBox(boxsurf, boxpos)
            
    def observer(self, result):
        self.logger.debug("observer called with %s" % result)
        if result:# correct card picked
            self.dbscore += (2 - self.WEcheat)
            self.actives.empty()
            self.EndExerciseFlag = True
            self.exercises -= 1
            self.displayedscore += 10 * 15
            self.levelupcount += 1
            self.scoredisplay.set_score(self.displayedscore)
            self.good_sound.play()
            self.good_image.display_sprite()
            pygame.time.wait(1500)
        else:
            self.dbscore -= 2
            if self.dbscore < 0:
                self.dbscore = 0
            self.displayedscore -= 10 * 50
            if self.displayedscore <= 0:
                self.displayedscore = 0
            self.levelupcount -= 1
            self.scoredisplay.set_score(self.displayedscore)
            self.wrong_sound.play()
            pygame.time.wait(1500)
            
    def _startbutton_cbf(self, sprite, *args):
        self.actives.remove(sprite)
        self.startbut.erase_sprite()
        if not self.WEcheat:
            for c in self.cardlist:
                c.close()
                c.change_image()
            if self.changepositions:
                positions = self.cardpositions[:len(self.cardlist)]
                random.shuffle(positions)
                for c in self.cardlist:
                    c.set_new_position(positions.pop())
        else:
            for c in self.cardlist:
                c.close()
                c.restart()
        self.pausesnd.play()
        pygame.time.wait(3500)
        
        for c in self.cardlist:
            c.open()
        self.actives.add(self.cardlist)
        self.actives.add(self.cheatbut)
        self.box.set_text(1)
        self.cheatbut.display_sprite()
    
    def _cheatbutton_cbf(self, sprite, *args):
        self.WEcheat = True
        self.actives.remove(sprite)
        self.cheatbut.erase_sprite()
        self.actives.empty()
        for c in self.cardlist:
            c.close()
            c.reset()
            pygame.time.wait(500)
        for c in self.cardlist:
            c.open()
        self.actives.add(self.startbut)
        self.box.set_text(2)
        self.startbut.display_sprite()        
    
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def get_moviepath(self):
        movie = os.path.join(self.my_datadir,'help.avi')
        return movie

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.redraw()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Ichanger")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "ichanger"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
                _("Several images from the past will appear on the screen.\nLook carefully at them and remember what you see."), 
        " ",
        _("Touch the 'START' button on the right.\nThe images will dissappear and when they return, one will have changed.\nTouch the picture which has changed."), 
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return []
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Miscellaneous")

    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 4

    def start(self):
        """Mandatory method."""
        self.SPG.tellcore_set_dice_minimal_level(3)
        self.startbut = SPWidgets.SimpleButtonDynamic(_('Start'), self.but_pos[0], fsize=24, colorname='green')
        self.startbut.set_use_current_background(True)
        self.cheatbut = SPWidgets.SimpleButtonDynamic(_('Cheat'), self.but_pos[0], fsize=24, colorname='green')
        self.cheatbut.set_use_current_background(True)
        self.startbut.connect_callback(self._startbutton_cbf, MOUSEBUTTONDOWN)
        self.cheatbut.connect_callback(self._cheatbutton_cbf, MOUSEBUTTONDOWN)
        self.AreWeDT = False
        p = os.path.join(self.CPdatadir,'good_%s.png' % self.lang)
        if not os.path.exists(p):
            p = os.path.join(self.CPdatadir,'thumbs.png')
        self.ThumbsUp = SPSpriteUtils.MySprite(utils.load_image(p))
    
    def dailytraining_pre_level(self, level):
        """Mandatory method"""
        pass
        
    def dailytraining_next_level(self, level, dbmapper):
        """Mandatory method.
        This should handle the DT logic"""
        self.AreWeDT = True
        self.SPG.tellcore_disable_level_indicator()
        self.next_level(level, dbmapper)
        return True
        
    def pre_level(self, level):
        """Mandatory method"""
        pass
  
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        if level > 4:
            return False
        self.level = level
        self.levelupcount = 1
        self.dbscore = 0
        # number of exercises in one level
        self.exercises = int(self.rchash[self.theme]['exercises'])
        self.actives.empty()
        self.changepositions = False
        # let the cards open until user hits start, then close them for some time, replace
        # image on one of the cards and show them again.
        self.next_exercise()
        return True
    
    def next_exercise(self):
        self.logger.debug("next_exercise called, exercises = %s" % (self.exercises))
        self.clear_screen()
        self.box.set_text(0)
        
        self.WEcheat = False
        self.EndExerciseFlag = False
        if not self.exercises:
            return True
        if self.level in (1, 2):
            if self.level == 2:
                self.changepositions = True
            num = 3
        else:
            if self.level == 4:
                self.changepositions = True
            num = 4
        if len(self.imagepaths) < num+1:
            self.imagepaths = self.imagepaths_org[:]
            random.shuffle(self.imagepaths)
        self.cardlist = []
        i = 0
        for i in range(num):
            f = self.imagepaths.pop()
            img = utils.load_image(f)
            c = self.opencard.convert_alpha()
            ix, iy = img.get_size()
            cx, cy = c.get_size()
            x = (cx - ix) / 2
            y = (cy - iy) / 2
            c.blit(img, (x, y))
            card = Card(self.closedcard, c, self.cardpositions[i], self.observer, f)
            card.display_sprite()
            self.cardlist.append(card)
            i += 1
        img = utils.load_image(self.imagepaths.pop())
        c = self.opencard
        ix, iy = img.get_size()
        cx, cy = c.get_size()
        x = (cx - ix) / 2
        y = (cy - iy) / 2
        c.blit(img, (x, y))
        card = random.choice(self.cardlist)
        card.set_changed_image(c)
        self.actives.add(self.startbut)
        self.startbut.display_sprite()
        
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        pass 
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        m,s = timespend.split(':')
        seconds = int(m)*60 + int(s)
        # M = Maximum score (number of exercises * 2)
        # P = Users result in points
        # F = Fraction of total result related to points. (maximum result is 10)
        # F1 = Fraction of result related to time
        # C = multiplier
        # S = time spend in seconds
        M = float(int(self.my_rchash['exercises']) * 2)
        P = float(self.dbscore)
        F = 7.0
        F1 = 3.0
        C = 0.2
        S = float(seconds)
        points = max((F/100) * (P/(M / 100)), 1.0)
        time = min(F1 / ((S / (M * 2)) ** C), F1)
        self.logger.info("@scoredata@ %s level %s M %s P %s S %s points %s time %s" %\
                          (self.get_name(),self.level, M, P, S, points, time))
        score = points + time
        result = points + time
        return result
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        for event in events:
            if event.type in (MOUSEBUTTONDOWN, MOUSEMOTION):
                self.actives.update(event)
            if not self.actives and self.EndExerciseFlag:
                self.clear_screen()
                pygame.time.wait(2000)
                if self.next_exercise():
                    self.clear_screen()
                    self.ThumbsUp.display_sprite((180, 100))
                    pygame.time.wait(2000)
                    self.ThumbsUp.erase_sprite()
                    if self.levelupcount >= int(self.rchash[self.theme]['autolevel_value']):
                        levelup = 1
                    else:
                        levelup = 0
                    if self.AreWeDT:
                        self.SPG.tellcore_level_end(level=self.level)
                    else:
                        self.SPG.tellcore_level_end(store_db=True, \
                                            level=min(4, self.level), \
                                            levelup=levelup)
        return 
        



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Rene Dohmen <acidjunk@gmail.com> and Peter Govers <petergovers@gmail.com>
# Copyright (c) 2010 Stas Zykiewicz <stas.zytkiewicz@gmail.com>
#           languageDefinitions.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# leave this line intact as it is
_ = lambda x:x

################## Put your translatable words in this list ###################
words = [ _("Good!"), 
         _("Cheat"), 
         _("Start"), 
         _("Try again!"), 
         _("New word"), 
         _("New question"), 
         _("Other category"), 
         _("Did you enjoy this?"), 
         _("Yes"), 
         _("No"), 
         _("Continue movie"), 
         _("Next movie"), 
         _("Memory"), 
         _("New Games"), 
         _("Puzzle"), 
         _("Quiz"), 
         _("Training"),
         _("Short Term"), 
         _("Long Term"), 
         _("Daily Training") 
         ]

############################### End of word definitions #####################

# Don't change stuff below this line.

# text from dailytraing.xml which is used in DT endscreen
a = _("short-term")
b = _("long-term")
c =_("quiz")


if __name__ == '__main__':
    
    import codecs
    import gettext
    from SPConstants import LOCALEDIR 

    lang_nl = gettext.translation('seniorplay', LOCALEDIR, languages=['nl'])
    lang_fr = gettext.translation('seniorplay', LOCALEDIR, languages=['fr'])
    lang_de = gettext.translation('seniorplay', LOCALEDIR, languages=['de'])
    lang_sv = gettext.translation('seniorplay', LOCALEDIR, languages=['sv'])

    lines = ["Words from the languageDefinitions file"]
    sep = "-" * 80
    
    for lang in [lang_nl, lang_de, lang_fr, lang_sv]:
        lines.append(sep)
        s = lang.info()['language-team'].split('_')[0].split('language: ')[0]
        lines.append("Language code: %s" % s)
        lines.append(sep)
        lang.install()
        _ = lang.ugettext
        for word in words:
            lines.append("%s = %s" % (word, _(word)))
    
    f = codecs.open('languageDefinitions_output.txt', 'wt', 'utf-8')
    f.write('\n'.join(lines))
    f.close()




# Copyright (c) 2010 stas zytkiewicz stas.zytkiewicz@schoolsplay.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# provides a logging object
# All modules get the same logger so this must called asap

### example "application" code to use in your module
##import logging
##module_logger = logging.getLogger("childsplay.<name of your module>")
##logger.debug("debug message")
##logger.info("info message")
##logger.warn("warn message")
##logger.error("error message")
##logger.critical("critical message")
##logger.exception("exception message")

import os
from SPConstants import HOMEDIR
LOGPATH = os.path.join(HOMEDIR, "childsplay.log")

# remove old log
if os.path.exists(LOGPATH):
    try:
        os.remove(LOGPATH)
    except Exception, info:
        print "Failed to remove old log"
        print info
    else:
        print "removed old logpath"
        
# set loglevel, possible values:
# logging.DEBUG
# logging.INFO
# logging.WARNING
# logging.ERROR
# logging.CRITICAL
import logging, logging.handlers

def set_level(level):
    global CONSOLELOGLEVEL, FILELOGLEVEL
    lleveldict = {'debug':logging.DEBUG,
        'info':logging.INFO,
        'warning':logging.WARNING,
        'error':logging.ERROR,
        'critical':logging.CRITICAL}
    if not lleveldict.has_key(level):
        print "Invalid loglevel: %s, setting loglevel to 'debug'" % level
        llevel = lleveldict['debug']
    else:
        llevel = lleveldict[level]
    CONSOLELOGLEVEL = llevel
    FILELOGLEVEL = llevel

def start():
    global CONSOLELOGLEVEL, FILELOGLEVEL
    #create logger
    logger = logging.getLogger("childsplay")
    logger.setLevel(CONSOLELOGLEVEL)
    sqla_logger = logging.getLogger("sqlalchemy")
    #create console handler and set level
    ch = logging.StreamHandler()
    ch.setLevel(CONSOLELOGLEVEL)
    #create file handler and set level
    fh = logging.FileHandler(LOGPATH)
    fh.setLevel(FILELOGLEVEL)
    #create formatter
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    #add formatter to ch and fh
    ch.setFormatter(formatter)
    fh.setFormatter(formatter)
    #add ch and fh to logger
    logger.addHandler(ch)
    logger.addHandler(fh)
    sqla_logger.addHandler(fh)
    logger.info("File logger created: %s" % LOGPATH)
    
    # test
    module_logger = logging.getLogger("childsplay.SPLogging")
    module_logger.info("logger created, start logging")
    import Version
    module_logger.info("Starting childsplay version: %s" % Version.version)
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           seniorplay
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os, shlex, glob,shutil

if len(sys.argv) > 2:
    if sys.argv[1] == '--plainversion':
        from SPVersion import version
        print version
        if os.path.exists('/data/userdata/.schoolsplay.rc/tmp'):# btp production only
            shutil.rmtree('/data/userdata/.schoolsplay.rc/tmp',True)
        sys.exit(0)
    # construct proper restart command if we need to restart
    prog = "python %s " % os.path.join(os.getcwd(), " ".join(sys.argv))

print sys.argv

import subprocess
#import gc
#gc.set_debug(gc.DEBUG_COLLECTABLE | gc.DEBUG_UNCOLLECTABLE | gc.DEBUG_INSTANCES | gc.DEBUG_OBJECTS)
# first parse commandline options
from SPOptionParser import OParser
# if this doesn't bail out the options are correct and we continue with schoolsplay

op = OParser()
# this will return a class object with the options as attributes  
CMD_Options = op.get_options()

######## Here we add options for debugging #####
### Must be removed or at least discussed before release #####
#CMD_Options.loglevel = 'debug'

import SPLogging
SPLogging.set_level(CMD_Options.loglevel)
SPLogging.start()

#create logger, configuration of logger was done above
import logging
CPmodule_logger = logging.getLogger("childsplay.childsplay")
CPmodule_logger.debug("Created schoolsplay loggers")

import utils

BTPPID = None
BTPSTART = None
try:
    BTPPID = CMD_Options.kill_btp
    BTPSTART = CMD_Options.restart_btp
except AttributeError:
    CPmodule_logger.info("No kill_btp or restart_btp option given")
else:
    if BTPPID and BTPSTART:
        CPmodule_logger.warning("Found BTP pid, killing BTP with pid: %s" % BTPPID)
        subprocess.Popen("kill -9 %s" % BTPPID, shell=True)
        utils._remove_lock()
        BTPSTART = 'cd .. && ' + BTPSTART
    else:
        CPmodule_logger.info("No pid for BTP")

try:
    import sqlalchemy as sqla
except ImportError:
    CPmodule_logger.exception("No sqlalchemy package found")
    sys.exit(1)
else:
    if sqla.__version__ < '0.5.8':
        CPmodule_logger.error("Found sqlalchemy version %s" % sqla.__version__)
        CPmodule_logger.error("Your version of sqlalchemy is to old, please upgrade to version >= 0.6")
        sys.exit(1)
    CPmodule_logger.debug("using sqlalchemy %s" % sqla.__version__)

import sqlalchemy.orm as sqlorm

import time
import datetime

#CPmodule_logger.info("IMPORTANT READ THE FOLLOWING LINES")
#CPmodule_logger.info("For debugging purposes we run with some cmdline options hardcoded.")
#CPmodule_logger.info("These must be removed before releasing this to the real world")
#CPmodule_logger.info("Look at the top of this module for these options")
from SPConstants import *
sys.path.insert(0, BASEDIR)
sys.path.insert(0, os.path.join(BASEDIR, 'lib'))

if CMD_Options.loglevel == 'debug':
    CPmodule_logger.debug("Paths defined in SPConstants:")
    for v in ['ACTIVITYDATADIR', 'ALPHABETDIR', 'BASEDIR',\
               'DBASEPATH', 'HOMEDIR', 'HOMEIMAGES', 'HOME_DIR_NAME',\
               'LOCALEDIR', 'LOCKFILE']:
        CPmodule_logger.debug("%s > %s" % (v, eval(v)))

# Rudimentary language check, when it fails it sets the cmdline option to C                                                                                 
# which is available on every proper GNU/Linux system. We don't have yet a dbase 
# connection at this stage so we just check if the locale is properly formed.
# We don't care about windows of course that system locale handling sucks anyway.
loc = CMD_Options.lang
if loc.find('_') == -1 or loc.upper().find('utf8') == -1:
    pass
    
# Make sure we have xml files in a writeable place    
name = 'childsplay'
p = os.path.join(HOMEDIR, name)
if not os.path.exists(p):
    os.makedirs(p)
orgxmlfiles = glob.glob(os.path.join(THEMESPATH, name, '*.xml'))
existingxmlfiles = os.listdir(p)
for f in  XML_FILES_WE_MUST_HAVE:
    if f not in existingxmlfiles and os.path.exists(os.path.join(THEMESPATH, name, f)):
        src = os.path.join(THEMESPATH, name, f)
        CPmodule_logger.debug("Copying %s to %s" % (src, p))
        shutil.copy(src, p)

## We also want to have the default photo albums in a writable location
#p = os.path.join(HOMEDIR, 'braintrainer', 'DefaultAlbums')
#if not os.path.exists(p):
#    os.makedirs(p)
#orgalbums = [a for a in glob.glob(os.path.join(ACTDATADIR, 'PhotoalbumData', '*_Album_*')) if os.path.isdir(a)]
#existingxmlfiles = os.listdir(p)
#for f in orgalbums:
#    if os.path.basename(f) not in existingxmlfiles and os.path.exists(f):
#        shutil.copytree(f, os.path.join(p, os.path.basename(f)))

# for childsplay we must make sure we have a content dbase in sqlite3 format present.
# if we want to use an sqlite content dbase we must make sure the default one is in the proper location.
p = os.path.join(HOMEDIR, CMD_Options.theme, CONTENTDBASE)
if not os.path.exists(p):
    if os.path.exists(CONTENTDBASE):
        shutil.copy(CONTENTDBASE, p)
    else:
        from SPBasePaths import SHARELIBDATADIR
        shutil.copy(os.path.join(SHARELIBDATADIR, CONTENTDBASE), p)
                        
import pygame
## set a bigger buffer, seems that on win XP in conjuction with certain hardware
## the playback of sound is scrambled with the "normal" 1024 buffer.
###  XXXX this still sucks, signed or unsigned that's the question :-(
pygame.mixer.pre_init(22050, -16, 2, 2048)
pygame.init()


# this will return the tuple (lang,rtl=bool)
LANG = utils.set_locale(lang=CMD_Options.lang)

if CMD_Options.checklog:
    try:
        import SPlogCheck
    except (ImportError, utils.SPError):
        sys.exit(1)
    except utils.MyError:
        sys.exit(1)
    sys.exit(0)
        
if not utils._set_lock():
    sys.exit(1)
    
import SPMainCore

from SPgdm import GDMEscapeKeyException

# start the maincore, we only return here on an exit
CPmodule_logger.debug("Start logging")
CPmodule_logger.debug("commandline options: %s" % CMD_Options)
CPmodule_logger.debug("SPMainCore running from: %s" % SPMainCore)

from SPDataManagerCreateDbase import DbaseMaker
DEBUG = False
try:
    #This will setup the dbases and ORMS
    dbm = DbaseMaker(CMD_Options.theme, debug_sql=DEBUG)            
except (AttributeError, sqla.exceptions.SQLAlchemyError, utils.MyError), info:
    CPmodule_logger.exception("Failed to start the DBase, %s" % info)
    raise utils.MyError, info

mainscreen = None
abort = 0 
tellcore_error = False 
while not abort:
    restartme = False
    try:
        if not tellcore_error:
            # query for a session id
            try:
                orm = dbm.get_all_orms()['stats_session']
                content_engine, user_engine = dbm.get_engines()
                session = sqlorm.sessionmaker(bind=user_engine)()
            except Exception, info:
                CPmodule_logger.exception("Failed to get orm for stats_session: %s" % info)
                abort = True
                break
        orm = orm(datetime=datetime.datetime.now())
        session.add(orm)
        session.commit()
        orm = dbm.get_all_orms()['stats_session']
        result = session.query(orm).all()[-1]
        session_id = result.ID
        session.close()
        # there's no support for other resolutions then 800x600
        mcgui = SPMainCore.MainCoreGui(resolution=(800,600),\
                                        options=CMD_Options, dbmaker = dbm,\
                                        mainscr=mainscreen, error=tellcore_error, \
                                        session_id=session_id)
        mainscreen = mcgui.get_mainscreen()
        mcgui.start()
    except SPMainCore.MainEscapeKeyException:
        CPmodule_logger.info("User hits exit/escape...")
        tellcore_error = False 
        if CMD_Options.no_login or CMD_Options.user:
            # we have no login screen or the user was passed as a cmdline option so we exit
            #sys.exit(0)
            abort = True
            CPmodule_logger.info("nologin screen, clean exit")
        elif CMD_Options.theme == 'childsplay':
            CPmodule_logger.info("Theme is childsplay, clean exit")
            abort = True
        else:
            CPmodule_logger.info("restarting core.")
            restartme = True
    except GDMEscapeKeyException:
        CPmodule_logger.info("login screen, clean exit")
        tellcore_error = False 
        break
    except utils.RestartMeException:
        CPmodule_logger.info("GIT pull occurred, need to restart myself.")
        restartme = True
        abort = True
        tellcore_error = False 
    except (SystemExit, utils.StopmeException),status:
        if str(status) == '0':
            CPmodule_logger.info("systemexit, clean exit")
            abort = True
        else:
            CPmodule_logger.info("systemexit, not a clean exit")
            CPmodule_logger.info("restarting core.")
            tellcore_error = True 
            mcgui.call_foreign_observers()
    except utils.SPError, info:
        CPmodule_logger.error("Unrecoverable error, not a clean exit")
        CPmodule_logger.info("restarting core.")
        tellcore_error = True
        mcgui.call_foreign_observers()
    except Exception,status:        
        CPmodule_logger.exception("unhandled exception in toplevel, traceback follows:")
        CPmodule_logger.info("restarting core.")
        tellcore_error = True
        mcgui.call_foreign_observers()
try:
    mcgui.activity.stop_timer()
except Exception, info:
    CPmodule_logger.warning("Failed to stop activity timers")
    
CPmodule_logger.info("Seniorplay stopped.")

#from SPWidgets import Dialog
#
#try:
#    import SPlogCheck
#except (ImportError, utils.SPError):
#    text = _("Failed to parse the logfile, please contact the developers.\nMessage was: %s" % info)
#    dlg = Dialog(text, buttons=[_('OK')], title=_('Warning !'))
#    dlg.run()
#except utils.MyError, info:
#    text = "%s" % info
#    #dlg.run()
CPmodule_logger.debug("quiting pygame and waiting 0.5 seconds.")
pygame.quit()
time.sleep(0.5)
#if sys.platform == "linux2":
#    CPmodule_logger.info("Removing pyc files")
#    subprocess.Popen('find . -name "*.pyc" -exec rm {} \;',shell=True )


# BT+ specific stuff
if (CMD_Options.theme == 'braintrainer' and restartme) or (BTPPID and BTPSTART):
    restartme = False
    CPmodule_logger.info("respawing with :%s" % BTPSTART)
    pid = subprocess.Popen(BTPSTART, shell=True).pid
    CPmodule_logger.debug("launched Control Panel with pid %s" % pid)
    sys.exit()





# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Schooldsplay goodies
# SPGoodies is a class that is passed on to the activity constructor
# It contains references to useful stuff, callback functions from the core
# and observers from the core.
import SPConstants
import logging
import os
from BorgSingleton import Borg
import utils

class SPGoodies(Borg):
    """This class is a singleton because we want that the core and
    activity always uses the same object"""
    def __init__(self, core, screen, virtkb, lang, lock,cb_pre_level_end, \
        cb_level_end, cb_game_end, cb_info_dialog, \
        cb_display_execounter, cb_set_framerate, \
        cb_enable_dice,cb_dice_minimal_level,\
        theme_rc, theme, cb_disable_menubuttons, cb_enable_menubuttons, \
        cb_disable_level_indicator,cb_enable_level_indicator, \
            cb_disable_exit_button, cb_enable_exit_button, \
            cb_disable_score_button, cb_enable_score_button, 
            cb_hide_level_indicator, cb_show_level_indicator, \
            cb_get_orm, cb_register_core_observers):
        Borg.__init__(self)
        # references to stuff, use the methods to get them
        self.logger = logging.getLogger("childsplay.SPGoodies")
        self.screen = screen
        self.virtkb = virtkb
        self.pre_level_end = cb_pre_level_end
        self.level_end = cb_level_end
        self.game_end = cb_game_end
        if lang:
            if '.' in lang[0] and len(lang[0]) > 4:
                lang = (lang[0][:5], lang[1])
        else:
            lang = ('en_US.utf8', False)
        self.localesetting = lang
        self.lock = lock
        self.info_dialog = cb_info_dialog
        self.display_execounter = cb_display_execounter
        self.set_framerate = cb_set_framerate
        self.enable_dice = cb_enable_dice
        self.set_dice_minimal_level = cb_dice_minimal_level
        self.theme_rc = theme_rc
        self.theme = theme
        self.disable_menubuttons = cb_disable_menubuttons
        self.enable_menubuttons = cb_enable_menubuttons
        self.disable_level_indicator = cb_disable_level_indicator
        self.enable_level_indicator = cb_enable_level_indicator
        self.disable_exit_button = cb_disable_exit_button
        self.enable_exit_button = cb_enable_exit_button
        self.disable_score_button = cb_disable_score_button
        self.enable_score_button = cb_enable_score_button
        self.hide_level_indicator = cb_hide_level_indicator
        self.show_level_indicator = cb_show_level_indicator
        self.register_core_observer = cb_register_core_observers
        self._get_orm = cb_get_orm
        self.core = core
                
    # these methods are the observers that an activity can use to signal the core
    def tellcore_pre_level_end(self):
        """Use this to notify the core that the pre_level is ended.
        The core will call next_level on the activity."""
        apply(self.pre_level_end)
    def tellcore_level_end(self, store_db=None, level=1, levelup=False, no_question=False):
        """Use this to notify the core that the level is ended.
        The core will call next_level on the activity."""
        apply(self.level_end, (store_db, level, levelup, no_question))
    def tellcore_game_end(self, store_db=None):
        """Use this to notify the core that the game is ended.
        The core will start the menu and delete the activity."""
        raise utils.StopGameException, store_db
    def tellcore_info_dialog(self, text):
        """Use this to ask the core to display a info dialog with the text @text"""
        apply(self.info_dialog, (text, ))
    def tellcore_display_execounter(self, total, text=''):
        """Use this to display a exeCounter object in the menu bar. @total must
        be an integer indicating the total numebr of exercises."""
        return apply(self.display_execounter, (total, ))
    def tellcore_set_framerate(self, rate):
        """Use this to lower the framerate at which the loop runs.
        The maximum is 30 per minute to prevent running the CPU at 100%.
        The core resets the rate to 30 when loading a activity so you should call
        this in 'start' or 'next_level'.
        WARNING: only use this is you know why you want to use it.
        """
        apply(self.set_framerate, (rate, ))
    def tellcore_enable_dice(self, enable=True):
        """Use this to enable or disable the dice button.
        @enable is a bool, False to disable the dice, True to enable it.
        Defaults to True.
        """
        if enable:
            self.enable_level_indicator()
        else:
            self.disable_level_indicator()
    def tellcore_set_dice_minimal_level(self, level):
        """Use this to set the level indicator to a minimal level.
        The value must be the maximum number of levels minus one.
        You must call this in your start method otherwise the core will reset
        it to 1"""
        apply(self.set_dice_minimal_level, (level,))
    def tellcore_disable_menubuttons(self):
        """Use this to ask the core to disable it's menu buttons.
        This is needed when the activity wants to cover the menubar.
        Be aware that the activity must provide a bullet proof way for the user
        to quit the activity."""
        apply(self.disable_menubuttons)
    def tellcore_enable_menubuttons(self):
        """Use this to ask the core to enable it's menu buttons.
        This makes only sense if you have disabled them before."""
        apply(self.enable_menubuttons)
    def tellcore_disable_level_indicator(self):
        apply(self.disable_level_indicator)
    def tellcore_enable_level_indicator(self):
        apply(self.enable_level_indicator)
    def tellcore_disable_exit_button(self):
        apply(self.disable_exit_button)
    def tellcore_enable_exit_button(self):
        apply(self.enable_exit_button)
    def tellcore_disable_score_button(self):
        apply(self.disable_score_button)
    def tellcore_enable_score_button(self):
        apply(self.enable_score_button)    
    def tellcore_hide_level_indicator(self):
        apply(self.hide_level_indicator)
    def tellcore_show_level_indicator(self):
        apply(self.show_level_indicator)
        
    def tellcore_register_core_observer(self, obs):
        """You must register any observer methods that should be called in case of an error
        to clean up your mess. Your observer must be a method or funtion that takes no arguments.
        For example; the camera used by facerecognition is crappy and must be stopped manually"""
        apply(self.register_core_observer, (obs,))
    # these methods provide stuff needed by the activity.
    def get_screen(self):
        """get a references to the main SDL screen"""
        return self.screen
    def get_background(self):
        """get a references to the background surface"""
        return self.background
    def get_virtual_keyboard(self):
        """get a virtual keyboard object which is setup by the maincore.
        when we not in kioskmode this will return None so be sure to check
        the object returnt before trying to use it."""
        return apply(self.virtkb)
    def get_localesetting(self):
        """Returns the current locale as set by schoolsplay.
        The value is a tuple with the language code and a bool signaling if
        were in a RTL environment (Hebrew and Arabic)"""
        return self.localesetting
    def get_screenclip(self):
        """get a rect representing the area to be 'clipped'.
        This should be used to set a clip to the screen when your activity sets
        a background of it's own. You should set a screen clip to prevent
        your background from overlapping the SP background.
        The rect returnt is based on the current them used."""
        return self.screenrect
    def get_thread_lock(self):
        """Returns a reference to the thread lock setup by the core.
        This lock MUST be used when dealing with the timer object from
        the SP Timer module.
        """
        return self.lock
    def get_theme(self):
        """Returns the theme name or None for the default theme.
        """
        return self.theme
    def get_theme_rc(self):
        """Returns the dictionary with the contents of the theme.rc file"""
        return self.theme_rc
    def autolevelup(self):
        """Returns a boolean indicating if activities should do autolevelup"""
        return self.autolevelup
    def get_scoredisplay(self):
        """Returns a reference to the score display widget in the menubar."""
        return self.scoredisplay
    def get_infobutton(self):
        """Returns a reference to the infobutton widget in the menubar.
        Use this to place the button in your own eventloop if you want the functionality"""
        return self.infobutton
    def get_quitbutton(self):
        """Returns a reference to the quitbutton widget in the menubar.
        Use this to place the button in your own eventloop if you want the functionality"""
        return self.quitbutton
    def get_quitbutton_rect(self):
        """Returns the rect for the quit button which is located in the menubar.
        Use this if you activity starts it's own eventloop and you want to check
        if the user hits the menu exit button."""
        return self.quitbutton.get_button_rect()
    # dbase stuff
    def get_orm(self, tablename, dbase):
        """Returns a sqlalchemy ORM  for the table and a Session object which
        is connected to the proper engine.
        tablename must be a string with the name of the dbase table.
        dbase must be one of the strings, 'user' or 'content', indicating from 
        which dbase we want the table."""
        return self.dm.get_orm(tablename, dbase)
    def get_mapper(self, activity, dbase):
        return self.dm.get_mapper(activity, dbase)
    def get_current_user_id(self):
        return self.dm.get_user_id()
    def get_current_user_name(self):
        return self.dm.get_username()
    def get_user_id_by_loginname(self, username):
        return self.dm.get_user_id_by_loginname(username)
    def get_user_dbrow_by_loginname(self, username):
        return self.dm.get_user_dbrow_by_loginname(username)
    def get_current_user_dbrow(self):
        return self.get_user_dbrow_by_loginname(self.dm.current_user)
    def get_served_content_orm(self):
        return self.dm.get_served_content_orm()
    def get_served_content_mapper(self):
        return self.dm.get_served_content_mapper()
    def check_served_content(self, rows, game_theme, minimum=10, all_ids=None):
        return self.dm._check_already_served(rows, minimum, game_theme, all_ids)
    def get_table_data_userdb(self, table):
        return self.dm.get_table_data_userdb(table)
    def get_rckey(self, actname, key):
        return self.dm._get_rcrow(actname, key)
    def set_rckey(self, actname, key, value, comment):
        self.dm._set_rcrow(actname, key, value, comment)
    def update_rckey(self, actname, key, val):
        self.dm._update_rcrow(actname, key, val)
    def get_stats_hash(self):
        return self.stats_hash
    # paths related to the place where SP is currently installed
    def get_libdir_path(self):
        """path to the 'lib' directory were activities store there stuff"""
        return SPConstants.ACTIVITYDATADIR
    def get_absdir_path(self):
        """path to the alphabetsounds directory"""
        return SPConstants.ALPHABETDIR
    def get_absdir_loc_path(self):
        """path to the alphabetsounds directory for the current language"""
        absdir = self.get_absdir_path()
        lang = self.localesetting[0]
        soundlanguagedir = os.path.join(absdir, lang)
        if not os.path.exists(soundlanguagedir):
            if os.path.exists(os.path.join(absdir, lang[:2])):
                soundlanguagedir = os.path.join(absdir, lang[:2])
                self.logger.debug("Using alphabethsounds package %s" % soundlanguagedir)
            else:
                soundlanguagedir = os.path.join(absdir, 'en')
                self.logger.debug("No alphabethsounds found for %s, using english." % lang)
        return soundlanguagedir
    def get_base_dir(self):
        return SPConstants.BASEPATH
    def get_home_theme_path(self):
        return os.path.join(SPConstants.HOMEDIR, self.theme)
    




# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPDebugDialog.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import pygame
from pygame.constants import *
import utils
from SPConstants import *
import logging 
import SPWidgets
import Mail
import os
import subprocess
import glob

GITBRANCH = 'master'

class Debugscreen:
    def __init__(self, actname, milestone, screen):
        self.logger = logging.getLogger("childsplay.SPDebugDialog.Debugscreen")
        self.orgscreen = pygame.display.get_surface()
        self.act_name = actname
        self.currentmilestone = milestone
        self.screen = screen
        self.screenshotpath = self._make_screenshot()
        
        id_pos = (60, 0)
        txt_0_pos = (60, 450)
        short_descr_pos = (60, 80)
        long_descr_pos = (60, 120)
        milestone_pos = (60, 360)
        component_pos = (60, 400)
        prio_pos = (550, 360)
        assigned_pos = (270, 360)
        name_pos = (270, 400)
        txt_1_pos = (60, 40)
        but_pos = (40, 500)
        fsize12 = 11
        fsize14 = 14
        
        self.labels = []
        self.buttons = []
        self.entries = []
        
        self.buthash = utils.OrderedDict()
        self.buthash = {"Cancel":self.on_quit_clicked, \
                        "Mail log":self.on_mail_logfile_clicked,\
                         "Mail scrshot":self.on_mail_scrshot_clicked, \
                         "Mail log+scrshot":self.on_mail_all_clicked,\
                         "Update from GIT":self.on_git_pull_clicked}

        # labels
        self.txt0_lbl = SPWidgets.Label("Your IP, Mac address and the date will be included in the mail.", \
                                txt_0_pos, fsize12, transparent=True)
        self.labels.append(self.txt0_lbl)
        self.txt1_lbl = SPWidgets.Label("Entries marked with a '*' are required fields without it your mail will not be send.", \
                                txt_1_pos, fsize14, bgcol=ORANGERED1)
        self.labels.append(self.txt1_lbl)
        short_descr_lbl = SPWidgets.Label("* Give a short description of the problem (Min 10 chars):", \
                                short_descr_pos, fsize12, transparent=True)
        self.labels.append(short_descr_lbl)
        long_descr_lbl = SPWidgets.Label("* Give a proper description of the issue and steps to reproduce it (Min 50 chars):", \
                               long_descr_pos, fsize12, transparent=True)
        self.labels.append(long_descr_lbl)
        milestone_lbl = SPWidgets.Label("Milestone:", milestone_pos, fsize12, transparent=True)
        self.labels.append(milestone_lbl)
        component_lbl = SPWidgets.Label("Component:", component_pos, fsize12, transparent=True)
        self.labels.append(component_lbl)
        prio_lbl = SPWidgets.Label("Priority:", prio_pos, fsize12, transparent=True)
        self.labels.append(prio_lbl)
        assigned_lbl =SPWidgets.Label("Assigned to:", assigned_pos, fsize12, transparent=True)
        self.labels.append(assigned_lbl)
        name_lbl = SPWidgets.Label("* Your name (Min 2 chars):", name_pos, fsize12, transparent=True)
        self.labels.append(name_lbl)
        # entries
        self.short_descr_te = SPWidgets.TextEntry((short_descr_pos[0] + short_descr_lbl.get_sprite_width()+12, short_descr_pos[1]), \
                                    length=300, fsize=fsize12, border=1)
        self.entries.append(self.short_descr_te)
        self.long_descr_te = SPWidgets.TextEntryBox((long_descr_pos[0]+12, long_descr_pos[1]+24),\
                                    height=10, length=600, fsize=fsize12)
        self.entries += self.long_descr_te.get_actives()
        self.labels.append(self.long_descr_te)
        self.milestone_te = SPWidgets.TextEntry((milestone_pos[0] + milestone_lbl.get_sprite_width()+12, milestone_pos[1]), \
                                    length=100, message=self.currentmilestone, fsize=fsize12, border=1)
        self.entries.append(self.milestone_te)
        self.component_te = SPWidgets.TextEntry((component_pos[0] + component_lbl.get_sprite_width()+12, component_pos[1]), \
                                    length=100, fsize=fsize12, border=1)
        self.entries.append(self.component_te)
        self.prio_te = SPWidgets.TextEntry((prio_pos[0] + prio_lbl.get_sprite_width()+12, prio_pos[1]), \
                                    length=100, message='3', fsize=fsize12, border=1)
        self.entries.append(self.prio_te)
        self.assigned_te = SPWidgets.TextEntry((assigned_pos[0] + assigned_lbl.get_sprite_width()+12, assigned_pos[1]), \
                                    length=150, message='Rene', fsize=fsize12, border=1)
        self.entries.append(self.assigned_te)
        self.name_te = SPWidgets.TextEntry((name_pos[0] + name_lbl.get_sprite_width()+12, name_pos[1]), \
                                    length=150, fsize=fsize12, border=1)
        self.entries.append(self.name_te)
        # buttons
        for label, func in self.buthash.items():
            b = SPWidgets.Button(label, but_pos, fsize=16, padding=8, name=label)
            b.connect_callback(func, MOUSEBUTTONDOWN, label)
            self.buttons.append(b)
            but_pos = (but_pos[0] + b.get_sprite_width()+12, but_pos[1])
        
        self._build_screen()

    def _build_screen(self):
        self.scrclip = self.screen.get_clip()
        self.screen.set_clip((0, 0, 800, 600))
        self.dim = utils.Dimmer()
        self.dim.dim(darken_factor=160, color_filter=CORNFLOWERBLUE)
    
        self.surf = pygame.Surface((750, 550))
        self.surf.fill(GREY92)
        self.screen.blit(self.surf, (25, 25))
        pygame.display.update()
        for s in self.labels + self.buttons + self.entries:
            s.display_sprite()
            
    def get_actives(self):
        return self.buttons + self.entries
    
    def on_quit_clicked(self, *args):
        self.logger.debug("quit called")
        for s in self.labels + self.buttons + self.entries:
            s.erase_sprite()
        self.screen.blit(self.orgscreen, (0, 0))
        self.dim.undim()
        self.screen.set_clip(self.scrclip)
        pygame.display.update()
        return -2
        
    def on_mail_logfile_clicked(self, *args):
        self.logger.debug("mail_logfile called")
        return self._mail_logfile(logpath=os.path.expanduser(os.path.join('~', '.schoolsplay.rc', 'schoolsplay.log')))

    def on_mail_scrshot_clicked(self, *args):
        self.logger.debug("mail_scrshot called")
        
        return self._mail_logfile(imgpath=self.screenshotpath)
        
    def on_mail_all_clicked(self, *args):
        self.logger.debug("mail_all")
        return self._mail_logfile(logpath=os.path.expanduser(os.path.join('~', '.schoolsplay.rc', 'schoolsplay.log')), \
                            imgpath=self.screenshotpath)
        
    def on_git_pull_clicked(self, *args):
        self.logger.debug("git_pull called")
        self.screen.blit(self.surf, (25, 25))
        # If we have network troubles, the smtp client timesout in 10 seconds.
        line = _("Git is updating your code, please wait.....")
        s = utils.char2surf(line, fsize=24, fcol=DARKGREEN,bold=True)
        self.screen.blit(s, (50, 300))
        pygame.display.update()
        process = subprocess.Popen(['git checkout %s & git pull' % GITBRANCH], shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        txt = list(process.communicate())
        # we run a script which could contain stuff needed after a git update.
        self.logger.debug("Run scripts from post_pull directory, if any.")
        pp_txt = self._run_post_pull()
        self.logger.debug(pp_txt)
        txt += pp_txt
        result = -3
        txt.append("\nThe system will be restarted with the current commandline options.")
        txtlist = [s for s in txt if s]
        if len(txtlist) > 13:
            txtlist = txtlist[:-40]
        txt = '\n'.join(txtlist)
        dlg = SPWidgets.Dialog(txt, dialogwidth=500, buttons=["OK"], title='Information',fsize=8)
        dlg.run()
        self.on_quit_clicked()
        return result

    def _run_post_pull(self):
        try:
            import db_conf as dbrc
            rckind = 'db_conf'
        except ImportError:
            import db_dev as dbrc
            rckind = 'db_dev'
        rc_hash = dbrc.rc
        if rc_hash['default']['production']:
            kind = 'production'
        else:
            kind = 'develop'
        rc_hash['kind'] = kind
        
        cmd_list = []
        print rc_hash
        for k in rc_hash.keys():
            for c, v in rc_hash[k].items():
                cmd_list.append('--%s=%s' % (c,v))
        
        ppp = os.path.expanduser(os.path.join('~', '.schoolsplay.rc', 'post_pull'))
        if os.path.exists(ppp):
            f = open(ppp, 'r')
            scripts_done = [l[:-1] for l in f.readlines()]
            f.close()
        else:
            scripts_done = []
        scripts = glob.glob(os.path.join('.','post_pull', '*.sh'))
        scripts.sort()
        scripts_todo = []
        for s in scripts:
            if s not in scripts_done:
                scripts_todo.append(s)
        out = ['Running any new post_pull scripts:']
        scripts_todo.sort()
        f = open(ppp, 'a')
        for s in scripts_todo:
            self.logger.debug("Running post_pull script %s" % s)
            cmd = ' '.join(['sh %s' %s] + cmd_list)
            self.logger.debug("using cmdline options %s" % cmd)          
            process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
            txt = []
            for l in process.communicate():
                if l:
                    l = l.replace('\n',' ')
                    txt.append(l)
            out += txt
            # TODO check exit code
            f.write("%s\n" % s)
        f.close()
        return out

    def _make_screenshot(self):
        try:
            scr = pygame.display.get_surface()
            path = os.path.join(HOMEDIR, self.act_name + '.jpeg')
            pygame.image.save(scr, path)
            self.logger.info("Screenshot %s taken" % path)
        except Exception, info:
            self.logger.error("Failed to make screenshot %s" % info)
            text = _("Failed to make screenshot %s") % info
            dlg = SPWidgets.Dialog(text, title="ERROR !")
            dlg.run()
            raise utils.MyError, "failed to make a screenshot"
        return path

    def _get_entrydata(self):
        data = {}
        data['short'] = self.short_descr_te.get_text()
        data['long'] = '\n'.join(self.long_descr_te.get_text())
        data['milestone'] = self.milestone_te.get_text()
        data['component'] = self.component_te.get_text()
        data['prio'] = self.prio_te.get_text()
        data['assigned'] = self.assigned_te.get_text()
        data['name'] = self.name_te.get_text()
        return data
            

    def _validate_data(self, data):
        entries = []
        if len(data['short']) < 10:
            entries.append('Short description')
        if len(data['long']) < 50:
            entries.append('Long description')
        if len(data['name']) < 2:
            entries.append('Name')
        return ','.join(entries)

    def _mail_logfile(self, logpath='', imgpath=''):
        self.logger.debug("_mail_logfile called")
        data = self._get_entrydata()
        result = self._validate_data(data)
        if result:
            dlg = SPWidgets.Dialog("The following entries are empty or have to little text:\n%s" % result,dialogwidth=500,  \
                    buttons=["OK"], title='Error !')
            dlg.run()
            return
        
        self.screen.blit(self.surf, (25, 25))
        # If we have network troubles, the smtp client timesout in 10 seconds.
        line = _("Your email will be send, please wait.....")
        s = utils.char2surf(line, fsize=24, fcol=DARKGREEN,bold=True)
        self.screen.blit(s, (50, 300))
        pygame.display.update()
        
        try:
            Mail.mail(subject=data['short'], description=data['long'], \
                      component=data['component'], prio=data['prio'], \
                      assigned=data['assigned'], milestone=data['milestone'], \
                      name=data['name'], \
                      logpath=logpath, imgpath=imgpath)
        except Mail.SendmailError,err:
            self.activity_info_dialog(_("Failed to send the email. Error: %s") % err)
        return self.on_quit_clicked()

if __name__ == '__main__':
    
    import __builtin__
    __builtin__.__dict__['_'] = lambda x:x
    
    import SPLogging
    SPLogging.set_level('debug')
    SPLogging.start()
        
    import pygame
    from pygame.constants import *
    pygame.init()
    
    from SPSpriteUtils import SPInit
    from SPWidgets import Init
    def cbf(sprite, event, data):
        print 'cb called with sprite %s, event %s and data %s' % (sprite, event, data)
        print 'sprite name: %s' % sprite.get_name()
        print 'data is %s' % data
    
    scr = pygame.display.set_mode((800, 600))
    scr.fill(LIGHTSKYBLUE1)
    pygame.display.flip()
    back = scr.convert()
    actives = SPInit(scr, back)
    
    Init('braintrainer')

    db = Debugscreen('none_act', 'BT2.1', scr)
    actives.add(db.get_actives())
    runloop = 1 
    while runloop:
        pygame.time.wait(100)
        pygame.event.pump()
        events = pygame.event.get()
        for event in events:
            if event.type is KEYDOWN:
                if event.key == K_ESCAPE:
                    runloop = 0
                elif event.key == K_F1:
                    print "F1"
                
            result = actives.update(event)
            if result and result[0][1] == -2:
                runloop = False
                    
                    
                    
